<?php
/**
 * Copyright 2011 pixeltricks GmbH
 *
 * This file is part of RequirementsEngine.
 *
 * RequirementsEngine is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * RequirementsEngine is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package RequirementsEngine
 * @subpackage Base
 */

/**
 * Extends the Silverstripe Requirements class with additional functionality.
 * You can now use the Silverstripe template engine in your .css and .js files
 * that are loaded via "Requirements::themedCSS|javascript".
 * 
 * Additionally all those files get sutured in live mode, decreasing the amount
 * of requests to your webserver.
 * 
 * To provide variables for your files add them via "RequirementsEngine::registerCssVariable()"
 * or "RequirementsEngine::registerJsVariable()".
 * 
 * IMPORTANT:
 * ----------
 * Make sure to call "RequirementsEngine::combine_files_and_parse()" in your
 * page controller's "init()" method.
 * 
 * 
 * Example:
 *      In your page controller's init method:
 *          RequirementsEngine::registerCssVariable('myCssColor', '#ff0000');
 * 
 *      In your CSS file:
 *          color: {$myCssColor};
 *
 * @package RequirementsEngine
 * @subpackage Base
 * @author Sebastian Diel <sdiel@pixeltricks.de>
 * @since 25.11.2010
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @copyright 2010 pixeltricks GmbH
 */
class RequirementsEngine extends Requirements {
    
    /**
     * Contains all blocked files
     *
     * @var array
     */
    protected static $registeredBlockedFiles = array();
    
    /**
     * Contains all registered CSS files
     *
     * @var array
     */
    protected static $registeredCssFiles = array();
    
    /**
     * Contains all registered variables for CSS files
     *
     * @var array
     */
    protected static $registeredCssVariables = array();
    
    /**
     * Contains all registered head tags files
     *
     * @var array
     */
    protected static $registeredHeadTags = array();
    
    /**
     * Contains all registered themed CSS files
     *
     * @var array
     */
    protected static $registeredThemedCssFiles = array();
    
    /**
     * Contains all registered JS files
     *
     * @var array
     */
    protected static $registeredJsFiles = array();
    
    /**
     * Contains all registered variables for JS files
     *
     * @var array
     */
    protected static $registeredJsVariables = array();
    
    /**
     * Block the cached filename, so that you can overload existing files that are defined in a module that gets loaded later.
     *
     * @param string $filename The filename
     * 
     * @return void
     * 
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 06.12.2011
     */
    public static function blockCached($filename) {
        $preparedFilename = self::getCacheFolderRel() . self::prepare_filename_for_caching($filename);
        self::block($preparedFilename);
    }
    
    /**
     * Uses the Silverstripe template engine to parse Requirements files and
     * sutures them. Suturing only works in live mode though.
     *
     * @param string $combinedFileName Name of Targetfile
     * @param array  $files            Files to combine
     *
     * @return void
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 06.01.2011
     */
    public static function combine_files_and_parse($combinedFileName, $files) {
        $originalSFC = SSViewer::get_source_file_comments();
        SSViewer::set_source_file_comments(false);
        foreach ($files as $index => $filename) {
            $cachedFileName     = self::getCacheFolderRel() . self::prepare_filename_for_caching($filename);
            $cachedFileNameAbs  = self::getCacheFolder() . self::prepare_filename_for_caching($filename);
            $parsedFile         = '';

            if ( (@$_GET['flush'] == 'all')
                 || !file_exists($cachedFileNameAbs)
                 || Director::isDev()) {
                
                if (strpos($filename, '/index.php') === 0) {
                    $filename = substr($filename, 11);
                }
                $pathInfo       = pathinfo($filename);
                $fileToParse    = file_get_contents(Director::baseFolder() . '/' . $filename);
                $templateParser = new SSViewer_FromString($fileToParse);

                if (array_key_exists('extension', $pathInfo) &&
                     !empty($pathInfo['extension'])) {
                    
                     if (strtolower($pathInfo['extension']) == 'css') {
                        $parsedFile = $templateParser->process(new ArrayData(self::getRegisteredCssVariables()));
                     }
                     if (strtolower($pathInfo['extension']) == 'js') {
                        $parsedFile = $templateParser->process(new ArrayData(self::getRegisteredJsVariables()));
                     }
                     
                     file_put_contents($cachedFileNameAbs, $parsedFile);
                 } 
            }
            self::block($filename);
            $files[$index] = $cachedFileName;
        }
        
        self::combine_files($combinedFileName, $files);
        SSViewer::set_source_file_comments($originalSFC);
    }
    
    /**
     * Uses the Silverstripe template engine to parse Requirements files.
     *
     * @param string $combinedFileName Name of Targetfile
     * @param array  $files            Files to combine
     *
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 05.10.2011
     */
    public static function parse($combinedFileName, $files) {
        $originalSFC = SSViewer::get_source_file_comments();
        SSViewer::set_source_file_comments(false);
        foreach ($files as $index => $filename) {
            $pathInfo       = pathinfo($filename);
            $cachedFileName     = self::getCacheFolderRel() . self::prepare_filename_for_caching($filename);
            $cachedFileNameAbs  = self::getCacheFolder() . self::prepare_filename_for_caching($filename);
            $parsedFile         = '';
            $fileType           = '';

            if (array_key_exists('extension', $pathInfo) &&
                !empty($pathInfo['extension'])) {
                
                if (strtolower($pathInfo['extension']) == 'css') {
                    $fileType = 'css';
                }
                if (strtolower($pathInfo['extension']) == 'js') {
                    $fileType = 'js';
                 }
            }
            
            if ( (@$_GET['flush'] == 'all')
                 || !file_exists($cachedFileNameAbs)
                 || Director::isDev()) {
                
                $fileToParse    = file_get_contents(Director::baseFolder() . '/' . $filename);
                $templateParser = new SSViewer_FromString($fileToParse);

                if (array_key_exists('extension', $pathInfo) &&
                     !empty($pathInfo['extension'])) {
                    
                     if ($fileType == 'css') {
                        $parsedFile = $templateParser->process(new ArrayData(self::getRegisteredCssVariables()));
                     }
                     if ($fileType == 'js') {
                        $parsedFile = $templateParser->process(new ArrayData(self::getRegisteredJsVariables()));
                     }
                     
                     file_put_contents($cachedFileNameAbs, $parsedFile);
                 } 
            }
            self::block($filename);
            
            if ($fileType == 'css') {
                self::css($cachedFileName);
            }
            if ($fileType == 'js') {
                self::javascript($cachedFileName);
            }
            
            $files[$index] = $cachedFileName;
        }
        SSViewer::set_source_file_comments($originalSFC);
    }
    
    /**
     * Deletes all cache files from the cache directory.
     *
     * @param boolean $flushCssFiles Indicates wether to delete CSS cache files
     * @param boolean $flushJsFiles  Indicates wether to delete JS cache files
     * 
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 27.09.2011
     */
    public static function flushCache($flushCssFiles = true, $flushJsFiles = true) {
         $cacheFiles = scandir(self::getCacheFolder());
         
         foreach ($cacheFiles as $cacheFile) {
             $pathInfo = pathinfo($cacheFile);
             
             if (array_key_exists('extension', $pathInfo) &&
                 !empty($pathInfo['extension'])) {
                 
                 if (strtolower($pathInfo['extension']) == 'css' &&
                     $flushCssFiles) {
                     
                     unlink(self::getCacheFolder().$cacheFile);
                 }
                 if (strtolower($pathInfo['extension']) == 'js' &&
                     $flushJsFiles) {
                     
                     unlink(self::getCacheFolder().$cacheFile);
                 }
             }
         }
    }

    /**
     * Returns a file system name (e.g. replaces '/' by '_').
     *
     * @param string $filename Dateiname
     *
     * @return string
     * 
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 06.01.2011
     */
    public static function prepare_filename_for_caching($filename) {
        $pathInfo       = pathinfo(Director::baseFolder().'/'.$filename);
        $replacements   = array(
            '/' => '_',
        );
        foreach ($replacements as $search => $replace) {
            $filename = str_replace($search, $replace, $filename);
        }
        if (Director::isLive()) {
            $filename = md5($filename) . sha1($filename) . '.' . $pathInfo['extension'];
        }
        return $filename;
    }
    
    /**
     * handles all registered requirements
     * 
     * @return void
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 06.12.2012
     */
    public static function handleRegisteredFiles() {
        $baseJsFileName         = 'script.js';
        $baseCssFileName        = 'base.css';
        $contentCssFileName     = 'content.css';
        $contentEcCssFileName   = 'content.ec.css';
        $prioritiveFileName     = 'content.pr.css';
        $combinedFilesFolder    = self::backend()->getCombinedFilesFolder() . '/';
        $combinedFilesPath      = Director::baseFolder() . '/' . $combinedFilesFolder;
        
        foreach (self::$registeredBlockedFiles as $blockedFiles) {
            self::block($blockedFiles);
        }
        
        if ((Member::currentUserID() > 0 &&
             Member::currentUser()->isAdmin() &&
             @$_GET['flush'] == 'all') ||
            Director::isDev() ||
            !file_exists($combinedFilesPath . $baseCssFileName)) {
            
            // temporary hold preloaded css files to prevent combine changes by 
            // different pages
            $preloadedCssFiles  = Requirements::backend()->get_css();
            $prioritiveCssFiles = array();
            
            foreach (self::$registeredThemedCssFiles as $priority => $themedCssList) {
                foreach ($themedCssList as $themedCss) {
                    self::themedCSS($themedCss);
                    if ($priority > 0) {
                        $prioritiveCssFiles[] = $themedCss;
                    }
                }
            }

            foreach (self::$registeredCssFiles as $priority => $cssList) {
                foreach ($cssList as $css) {
                    self::css($css);
                    if ($priority > 0) {
                        $prioritiveCssFiles[] = basename($css, '.css');
                    }
                }
            }
            foreach (self::$registeredJsFiles as $js) {
                self::javascript($js);
            }

            $contentCssFiles = array(
                'content',
                'forms',
                'patch_forms',
            );

            $combinedCssFiles           = array();
            $combinedContentCssFiles    = array();
            $combinedSilvercartCssFiles = array();
            $combinedPrioritiveCssFiles = array();
            // combine the themed css files here into different arrays
            foreach (self::backend()->get_css() as $file => $value) {
                if (array_key_exists($file, $preloadedCssFiles)) {
                    // skip preloaded css files to prevent combine changes by different pages
                    continue;
                }
                if (in_array(basename($file, '.css'), $prioritiveCssFiles)) {
                    $combinedPrioritiveCssFiles[] = $file;
                } elseif (strpos(basename($file), 'Silvercart') === 0) {
                    $combinedSilvercartCssFiles[] = $file;
                } elseif (in_array(basename($file, '.css'), $contentCssFiles)) {
                    $combinedContentCssFiles[] = $file;
                } else {
                    $combinedCssFiles[] = $file;
                }
            }

            $combinedJsFiles = array();
            foreach (self::backend()->get_javascript() as $file) {
                $combinedJsFiles[] = $file;
            }


            SilvercartThemeConfig::registerVariables();
            self::registerCssVariable('CurrentController',          Controller::curr());
            self::combine_files($baseJsFileName,                    $combinedJsFiles);
            self::combine_files_and_parse($baseCssFileName,         $combinedCssFiles);
            self::combine_files_and_parse($contentCssFileName,      $combinedContentCssFiles);
            self::combine_files_and_parse($contentEcCssFileName,    $combinedSilvercartCssFiles);
            if (count($combinedPrioritiveCssFiles) > 0) {
                self::combine_files_and_parse($prioritiveFileName,    $combinedPrioritiveCssFiles);
            }
            self::process_combined_files();
        } else {
            self::css($combinedFilesFolder . $baseCssFileName);
            self::css($combinedFilesFolder . $contentCssFileName);
            self::css($combinedFilesFolder . $contentEcCssFileName);
            self::css($combinedFilesFolder . $prioritiveFileName);
            self::javascript($combinedFilesFolder . $baseJsFileName);
        }
        foreach (self::$registeredHeadTags as $key => $headTag) {
            self::insertHeadTags($headTag, $key);
        }
    }

    /**
     * Registers a blocked file
     *
     * @param string $fileName The name of the file to register
     *
     * @return void
     * 
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 06.12.2012
     */
    public static function registerBlockedFile($fileName) {
        if (!in_array($fileName, self::$registeredBlockedFiles)) {
            self::$registeredBlockedFiles[] = $fileName;
        }
    }

    /**
     * Registers a CSS file
     *
     * @param string $fileName The name of the file to register
     * @param int    $priority Prioority of the file
     *
     * @return void
     * 
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 06.12.2012
     */
    public static function registerCssFile($fileName, $priority = 0) {
        if (!array_key_exists($priority, self::$registeredCssFiles)) {
            self::$registeredCssFiles[$priority] = array();
        }
        if (!in_array($fileName, self::$registeredCssFiles)) {
            self::$registeredCssFiles[$priority][] = $fileName;
        }
    }

    /**
     * Registers a variable for CSS templates
     *
     * @param string $variableName  The name of the variable
     * @param mixed  $variableValue The value of the variable
     *
     * @return void
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 27.09.2011
     */
    public static function registerCssVariable($variableName, $variableValue) {
        self::$registeredCssVariables[$variableName] = $variableValue;
    }

    /**
     * Registers a variable for CSS templates
     *
     * @param string $headTag The head tag to register
     * @param string $key     The key to identify the head tag
     *
     * @return void
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 27.09.2011
     */
    public static function registerHeadTag($headTag, $key) {
        self::$registeredHeadTags[$key] = $headTag;
    }

    /**
     * Registers a themed CSS file
     *
     * @param string $fileName The name of the file to register
     * @param int    $priority Prioority of the file
     *
     * @return void
     * 
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 06.12.2012
     */
    public static function registerThemedCssFile($fileName, $priority = 0) {
        if (!array_key_exists($priority, self::$registeredThemedCssFiles)) {
            self::$registeredThemedCssFiles[$priority] = array();
        }
        if (!in_array($fileName, self::$registeredThemedCssFiles)) {
            self::$registeredThemedCssFiles[$priority][] = $fileName;
        }
    }
    
    /**
     * Registers a JS file
     *
     * @param string $fileName The name of the file to register
     *
     * @return void
     * 
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 06.12.2012
     */
    public static function registerJsFile($fileName) {
        if (!in_array($fileName, self::$registeredJsFiles)) {
            self::$registeredJsFiles[] = $fileName;
        }
    }
    
    /**
     * Registers a variable for JS templates
     *
     * @param string $variableName  The name of the variable
     * @param mixed  $variableValue The value of the variable
     *
     * @return void
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 27.09.2011
     */
    public static function registerJsVariable($variableName, $variableValue) {
        self::$registeredJsVariables[$variableName] = $variableValue;
    }
    
    /**
     * Returns all registered CSS variables as associative array.
     *
     * @return array
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 27.09.2011
     */
    public static function getRegisteredCssVariables() {
        return self::$registeredCssVariables;
    }
    
    /**
     * Returns all registered JS variables as associative array.
     *
     * @return array
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 27.09.2011
     */
    public static function getRegisteredJsVariables() {
        return self::$registeredJsVariables;
    }
    
    /**
     * Returns the absolute path to the folder for cache files.
     *
     * @return string
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 27.09.2011
     */
    protected static function getCacheFolder() {
        return Director::baseFolder().'/'.self::getCacheFolderRel();
    }
    
    /**
     * Returns the relative path to the folder for cache files.
     *
     * @return string
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 12.10.2011
     */
    protected static function getCacheFolderRel() {
        return 'requirements_engine/cache/';
    }
}