<?php
/**
 * Copyright 2012 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package Silvercart
 * @subpackage Rating
 */

/**
 * A customer rating related to a product
 *
 * @package Silvercart
 * @subpackage Rating
 * @author Sebastian Diel <sdiel@pixeltricks.de>
 * @copyright 2012 pixeltricks GmbH
 * @since 23.03.2012
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 */
class SilvercartCustomerRating extends DataObject {
    
    /**
     * DB attributes
     *
     * @var array
     */
    public static $db = array(
        'Rating'        => 'Enum("1,2,3,4,5")',
        'Author'        => 'Varchar(64)',
        'Message'       => 'Text',
        'Locale'        => 'DBLocale',
    );

    /**
     * 1-1 and 1-many relations
     *
     * @var array
     */
    public static $has_one = array(
        'SilvercartProduct' => 'SilvercartProduct',
    );
    
    /**
     * Default sort
     *
     * @var string 
     */
    public static $default_sort = "Created DESC";
    
    /**
     * Cached/marked product to use after deletion
     *
     * @var SilvercartProduct
     */
    protected $cachedProduct = null;

    /**
     * Field labels for display in tables.
     *
     * @param boolean $includerelations A boolean value to indicate if the labels returned include relation fields
     *
     * @return array
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 23.03.2012
     */
    public function fieldLabels($includerelations = true) {
        $fieldLabels = array_merge(
            parent::fieldLabels($includerelations),
            array(
                'Rating'        => _t('SilvercartCustomerRating.RATING'),
                'Author'        => _t('SilvercartCustomerRating.AUTHOR'),
                'Message'       => _t('SilvercartCustomerRating.MESSAGE'),
            )
        );

        $this->extend('updateFieldLabels', $fieldLabels);
        return $fieldLabels;
    }

    /**
     * Returns the translated plural name of the object. If no translation exists
     * the class name will be returned.
     *
     * @return string the objects plural name
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 20.07.2012
     */
    public function plural_name() {
        return SilvercartTools::plural_name_for($this);
    }

    /**
     * Returns the translated singular name of the object. If no translation exists
     * the class name will be returned.
     *
     * @return string The objects singular name
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 20.07.2012
     */
    public function singular_name() {
        return SilvercartTools::singular_name_for($this);
    }

    /**
     * Summaryfields for display in tables.
     *
     * @return array
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 20.07.2012
     */
    public function summaryFields() {
        $summaryFields = array(
                'Rating'    => $this->fieldLabel('Rating'),
                'Author'    => $this->fieldLabel('Author'),
                'Message'   => $this->fieldLabel('Message'),
        );

        $this->extend('updateSummaryFields', $summaryFields);
        return $summaryFields;
    }
    
    /**
     * calculates the average rating of product when a new rating is added
     * 
     * @return void
     * 
     * @author Patrick Schneider <pschneider@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 07.09.2012
     */
    public function onAfterWrite() {
        parent::onAfterWrite();
        $this->recalculateCustomerRating();
    }
    
    /**
     * marks the related product for recalculating its average rating values
     * 
     * @return void
     * 
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 07.09.2012
     */
    protected function onBeforeDelete() {
        parent::onBeforeDelete();
        $this->cachedProduct = $this->SilvercartProduct();
    }
    
    /**
     * calculates the average rating of product when a rating is deleted
     * 
     * @return void
     * 
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 07.09.2012
     */
    protected function onAfterDelete() {
        parent::onAfterDelete();
        $this->recalculateCustomerRating($this->cachedProduct);
    }
    
    /**
     * calculates the average rating of product
     * 
     * @param SilvercartProduct $product The product to change rating data for
     * 
     * @return void
     * 
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 07.09.2012
     */
    public function recalculateCustomerRating($product = null) {
        if (is_null($product)) {
            $product = $this->SilvercartProduct();
        }
        $ratings = DataObject::get(
                'SilvercartCustomerRating',
                sprintf(
                        "`SilvercartCustomerRating`.`SilvercartProductID` = '%s'",
                        $product->ID
                )
        );
        $avarageCustomerRating  = 0;
        $customerRatingCount    = 0;
        if ($ratings) {
            $customerRatingCount = $ratings->Count();
            foreach ($ratings as $rating) {
                $avarageCustomerRating += $rating->Rating;
            }
        }
        if ($avarageCustomerRating > 0) {
            $avarageCustomerRating = round($avarageCustomerRating / $ratings->Count(), 2);
        }
        $product->AvarageCustomerRating   = $avarageCustomerRating;
        $product->CustomerRatingCount     = $customerRatingCount;
        $product->write();
    }
    
    /**
     * Returns the Created date in locale context.
     * Requires PHP 5.3 or higher to use IntlDateFormatter
     *
     * @return string
     */
    public function getCreatedForLocale() {
        $createdForLocale = '';
        if (class_exists('IntlDateFormatter')) {
            $date   = strtotime($this->Created);
            $fmt    = new IntlDateFormatter(
                    Translatable::get_current_locale(),
                    IntlDateFormatter::FULL,
                    IntlDateFormatter::NONE
            );
            $createdForLocale = $fmt->format($date);
        } else {
            $date = $this->dbObject('Created');
            $createdForLocale = $date->Nice();
        }
        return $createdForLocale;
    }
    
    /**
     * Returns whether this rating is a review
     *
     * @return boolean 
     * 
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 07.05.2012
     */
    public function isReview() {
        $isReview = false;
        if (!empty($this->Message)) {
            $isReview = true;
        }
        return $isReview;
    }
}
