<?php
/**
 * Copyright 2012 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package Silvercart
 * @subpackage Rating
 */

/**
 * Extension to relate a product with many customer ratings
 *
 * @package Silvercart
 * @subpackage Rating
 * @author Sebastian Diel <sdiel@pixeltricks.de>
 * @copyright 2012 pixeltricks GmbH
 * @since 23.03.2012
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 */
class SilvercartCustomerRatingProduct extends DataObjectDecorator {
    
    /**
     * Extra statics for the decorated DataObject
     * 
     * @return void
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 23.03.2012
     */
    public function extraStatics() {
        return array(
            'db'        => array(
                'AvarageCustomerRating' => 'Float',
                'CustomerRatingCount'   => 'Int',
            ),
            'has_many'  => array(
                'SilvercartCustomerRatings' => 'SilvercartCustomerRating',
            ),
        );
    }
    
    /**
     * Updates the field labels
     *
     * @param array &$labels Labels to update
     * 
     * @return void
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 23.03.2012
     */
    public function updateFieldLabels(&$labels) {
        $labels = array_merge(
                $labels,
                array(
                    'SilvercartCustomerRatings' => _t('SilvercartCustomerRating.PLURALNAME'),
                )
        );
    }
    
    /**
     * returns all ratings related to the product
     * 
     * @return DataObjectSet
     * 
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 07.05.2012 
     */
    public function SilvercartCustomerReviews() {
        $ratings        = new DataObjectSet();
        $groupedRatings = $this->owner->SilvercartCustomerRatings()->groupBy('Locale');
        if (array_key_exists(Translatable::get_current_locale(), $groupedRatings)) {
            foreach ($groupedRatings[Translatable::get_current_locale()] as $rating) {
                if ($rating->isReview()) {
                    $ratings->push($rating);
                }
            }
        }
        return $ratings;
    }
    
    /**
     * Returns the customre rating to use for css classes
     *
     * @return string
     */
    public function getAvarageCustomerRatingForCSS() {
        $simpleAvarageCustomerRating = round($this->owner->AvarageCustomerRating * 2) / 2;
        return str_replace('.', '_', $simpleAvarageCustomerRating);
    }
    
    /**
     * Returns the count of X star ratings
     *
     * @param int $starCount Count of stars to get count for
     * 
     * @return int
     */
    public function getSilvercartCustomerRatingCountFor($starCount) {
        if ($this->owner->SilvercartCustomerRatings()->Count() == 0) {
            $count = 0;
        } else {
            $groupedRatings = $this->owner->SilvercartCustomerRatings()->groupBy('Rating');
            if (!array_key_exists($starCount, $groupedRatings)) {
                $count = 0;
            } else {
                $count = count($groupedRatings[$starCount]);
            }
        }
        return $count;
    }
    
    /**
     * Returns the percentage of X star ratings
     *
     * @param int $starCount Count of stars to get percentage for
     * 
     * @return int
     */
    public function getSilvercartCustomerRatingPercentageFor($starCount) {
        if ($this->owner->SilvercartCustomerRatings()->Count() == 0) {
            $percentage = 0;
        } else {
            $count = $this->getSilvercartCustomerRatingCountFor($starCount);
            if ($count == 0) {
                $percentage = 0;
            } else {
                $percentage = floor($count / ($this->owner->SilvercartCustomerRatings()->Count() / 100));
            }
        }
        return $percentage;
    }
    
}