<?php
/**
 * Copyright 2011 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package SilvercartProductVariants
 * @subpackage Backend
 */

/**
 * Manages variant attribute sets.
 *
 * @package SilvercartProductVariants
 * @subpackage Backend
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @since 10.11.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @copyright 2011 pixeltricks GmbH
 */
class SilvercartProductVariantAttributeSetAdmin extends ModelAdmin {

    /**
     * The code of the menu under which this admin should be shown.
     * 
     * @var string
     */
    public static $menuCode = 'products';

    /**
     * The section of the menu under which this admin should be grouped.
     * 
     * @var string
     */
    public static $menuSortIndex = 11;

    /**
     * The URL segment
     *
     * @var string
     */
    public static $url_segment = 'silvercart-product-variants';

    /**
     * The menu title
     *
     * @var string
     */
    public static $menu_title = 'Silvercart product variant sets';

    /**
     * Managed models
     *
     * @var array
     */
    public static $managed_models = array(
        'SilvercartProductVariantAttributeSet' => array(
            'collection_controller' => 'SilvercartProductVariantAttributeSetAdmin_CollectionController',
            'record_controller'     => 'SilvercartProductVariantAttributeSetAdmin_RecordController'
        ),
        'SilvercartProductVariantAttribute' => array(
            'collection_controller' => 'SilvercartProductVariantAttributeAdmin_CollectionController'
        ),
        'SilvercartProductVariantConfiguration' => array(
            'enableFirstEntryAutoLoad'  => true
        ),
    );

    /**
     * Constructor
     *
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 10.07.2012
     */
    public function __construct() {
        self::$menu_title = _t('SilvercartProductVariantProduct.TAB_VARIANT_LABEL');

        parent::__construct();
    }
}

/**
 * Manages variant attribute sets.
 *
 * @package SilvercartProductVariants
 * @subpackage Backend
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @since 10.11.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @copyright 2011 pixeltricks GmbH
 */
class SilvercartProductVariantAttributeSetAdmin_CollectionController extends ModelAdmin_CollectionController {
    
}

/**
 * Manages variant attribute sets.
 *
 * @package SilvercartProductVariants
 * @subpackage Backend
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @since 13.01.2012
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @copyright 2012 pixeltricks GmbH
 */
class SilvercartProductVariantAttributeSetAdmin_RecordController extends SilvercartManyManyOrderField_RecordController {
    
    /**
     * Adds the abillity to execute additional actions to the model admin's
     * action handling.
     *
     * @param SS_HTTPRequest $request The request object
     * 
     * @return mixed
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 13.01.2012
     */
    public function handleAction(SS_HTTPRequest $request) {
        $allRequestParams = $request->allParams();
        
        if (array_key_exists('ID', $allRequestParams)) {
            $action = str_replace('action_', '', $allRequestParams['ID']);
        }
        if (array_key_exists('OtherID', $allRequestParams)) {
            $otherId = $allRequestParams['OtherID'];
        }
        
        switch ($action) {
            case 'doUpdateAttributedSets':
                return $this->doUpdateAttributedSets($request, $otherId);
                break;
            default:
                return parent::handleAction($request);
        }
    }
    
    /**
     * Updates attributed variant sets.
     *
     * @param SS_HTTPRequest $request        The request object
     * @param int            $attributeSetId The ID of the attribute set
     * 
     * @return void
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 13.01.2012
     */
    public function doUpdateAttributedSets(SS_HTTPRequest $request, $attributeSetId) {
        $statusMessage          = _t('SilvercartProductVariantAttributeSetAdmin.STATUS_MESSAGE');
        $attributeSet           = DataObject::get_by_id('SilvercartProductVariantAttributeSet', $attributeSetId);
        $deletedAttributesCount = 0;
        $addedAttributesCount   = 0;

        if ($attributeSet) {
            $attributedAttributeSets = DataObject::get( 'SilvercartAttributedVariantAttributeSet',
                sprintf(
                    "SilvercartProductVariantAttributeSetID = %d",
                    $attributeSetId
                )
            );

            if ($attributedAttributeSets) {
                foreach ($attributedAttributeSets as $attributedAttributeSet) {
                    $attributedAttributeSet->setField('SortOrder', $attributedAttributeSet->SilvercartProductVariantAttributeSet()->getField('SortOrder'));
                    $attributedAttributeSet->write();

                    // Delete old attributes
                    $deleteAttributes = array();

                    foreach ($attributedAttributeSet->Attributes() as $attributedAttribute) {
                        if (!$attributeSet->Attributes()->find('ID', $attributedAttribute->ID)) {
                            $deleteAttributes[] = $attributedAttribute->ID;
                        }
                    }

                    foreach ($deleteAttributes as $deleteAttribute) {
                        $attributedAttributeSet->Attributes()->remove($deleteAttribute);
                        $deletedAttributesCount++;
                    }

                    // Add new attributes
                    foreach ($attributeSet->Attributes() as $attribute) {
                        if (!$attributedAttributeSet->Attributes()->find('ID', $attribute->ID)) {
                            $attributedAttributeSet->Attributes()->add($attribute);
                            $addedAttributesCount++;
                        }
                    }
                }
            }
        }

        $statusMessage = sprintf($statusMessage, $deletedAttributesCount, $addedAttributesCount);
        return $this->returnVariantActionsFieldForAjaxRequests($statusMessage, $attributeSetId);
    }

    /**
     * Returns the field for ajax requests.
     * 
     * @param string $statusMessage  The status message
     * @param int    $attributeSetId The ID of the attribute set
     *
     * @return string
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 13.01.2012
     */
    public function returnVariantActionsFieldForAjaxRequests($statusMessage, $attributeSetId) {
        $data         = array();
        $attributeSet = DataObject::get_by_id('SilvercartProductVariantAttributeSet', $attributeSetId);

        $silvercartVariantActionsField = new SilvercartVariantActionsField(
            $attributeSet,
            'variantActions',
            _t('SilvercartProductVariantAttributeSet.ACTIONS_FOR_SET'),
            $statusMessage
        );

        return $silvercartVariantActionsField->customise($data)->renderWith('SilvercartVariantActionsField');
    }
}
