<?php
/**
 * Copyright 2011 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package SilvercartProductVariants
 * @subpackage Base
 */

/**
 * An attribute set containing many attributes that can be (de)activated; it's
 * attached to a product.
 *
 * @package SilvercartProductVariants
 * @subpackage Base
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @since 22.11.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @copyright 2011 pixeltricks GmbH
 */
class SilvercartAttributedVariantAttributeSet extends DataObject {
    
    /**
     * The default sort order.
     *
     * @var string
     */
    public static $defaul_sort = 'SortOrder ASC';

    /**
     * Attributes
     *
     * @var array
     */
    public static $db = array(
        'useCustomFormField' => 'VarChar(255)',
        'SortOrder'          => 'Int'
    );
    
    /**
     * Has-one relationships.
     *
     * @var array
     */
    public static $has_one = array(
        'SilvercartProduct'                     => 'SilvercartProduct',
        'SilvercartProductVariantAttributeSet'  => 'SilvercartProductVariantAttributeSet'
    );
    
    /**
     * Many-many relationships.
     *
     * @var array
     */
    public static $many_many = array(
        'Attributes' => 'SilvercartProductVariantAttribute'
    );
    
    /**
     * Many-many extra fields.
     *
     * @var array
     */
    public static $many_many_extraFields = array(
        'Attributes' => array(
            'isActive'  => 'Boolean(0)',
            'isDefault' => 'Boolean(0)'
        )
    );
    
    /**
     * Returns the translated singular name of the object. If no translation exists
     * the class name will be returned.
     * 
     * @return string The objects singular name 
     * 
     * @author Roland Lehmann <rlehmann@pixeltricks.de>
     * @since 14.07.2012
     */
    public function singular_name() {
        return SilvercartTools::singular_name_for($this);
    }


    /**
     * Returns the translated plural name of the object. If no translation exists
     * the class name will be returned.
     * 
     * @return string the objects plural name
     * 
     * @author Roland Lehmann <rlehmann@pixeltricks.de>
     * @since 14.07.2012
     */
    public function plural_name() {
        return SilvercartTools::plural_name_for($this); 
    }
    
    /**
     * Summaryfields for display in tables.
     *
     * @return array
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 23.11.2011
     */
    public function summaryFields() {
        $summaryFields = array(
            'SilvercartProduct.Title'                   => _t('SilvercartProduct.COLUMN_TITLE'),
            'SilvercartProductVariantAttributeSet.name' => _t('SilvercartProductVariantAttributeSet.NAME_LABEL')
        );
        
        $this->extend('updateSummaryFields', $summaryFields);
        return $summaryFields;
    }

    /**
     * Returns a sorted list of attributes for this set.
     *
     * @return DataObjectSet
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 11.03.2012
     */
    public function getAttributes() {
        $attributeSet = new DataObjectSet();
        $records      = DB::query(
            sprintf(
                "
                SELECT
                    AttASet_Att.SilvercartProductVariantAttributeID,
                    AttASet_Att.isActive,
                    AttASet_Att.isDefault,
                    ASet_Att.SortOrder
                FROM
                    SilvercartAttributedVariantAttributeSet_Attributes AttASet_Att
                LEFT JOIN
                    SilvercartProductVariantAttributeSet_Attributes ASet_Att
                ON
                    (ASet_Att.SilvercartProductVariantAttributeSetID = %d AND
                     ASet_Att.SilvercartProductVariantAttributeID = AttASet_Att.SilvercartProductVariantAttributeID)
                WHERE
                    AttASet_Att.SilvercartAttributedVariantAttributeSetID = %d
                ",
                $this->SilvercartProductVariantAttributeSet()->ID,
                $this->ID
            )
        );

        foreach ($records as $record) {
            $attribute = DataObject::get_by_id('SilvercartProductVariantAttribute', $record['SilvercartProductVariantAttributeID']);
            if ($attribute) {
                $attribute->isActive  = $record['isActive'];
                $attribute->isDefault = $record['isDefault'];
                $attribute->SortOrder = $record['SortOrder'];
                $attributeSet->push($attribute);
            }
        }

        $attributeSet->sort('SortOrder', 'ASC');

        return $attributeSet;
    }

    /**
     * Customize scaffolding fields for the backend
     *
     * @return FieldSet the form fields for the backend
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 31.01.2012
     */
    public function getCMSFields() {
        $fields = parent::getCMSFields();

        $this->extend('updateCMSFields', $fields);
    
        return $fields;
    }
}
