<?php
/**
 * Copyright 2011 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package SilvercartProductVariants
 * @subpackage Base
 */

/**
 * Depicts an attribute of a product that is purely informational, i.e. it
 * doesn't alter any product related attribute like price, product number, etc.
 *
 * @package SilvercartProductVariants
 * @subpackage Base
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @since 10.11.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @copyright 2011 pixeltricks GmbH
 */
class SilvercartProductVariantAttribute extends DataObject {

    /**
     * Castings.
     *
     * @var array
     */
    public static $casting = array(
        'name' => 'VarChar(255)'
    );

    /**
     * Has-many relationships.
     *
     * @var array
     */
    public static $has_many = array(
        'SilvercartProductVariantAttributeLanguages'    => 'SilvercartProductVariantAttributeLanguage',
        'SilvercartProductVariantFieldModifiers'        => 'SilvercartProductVariantFieldModifier',
    );

    /**
     * Belongs-many-many relationships.
     *
     * @var array
     */
    public static $belongs_many_many = array(
        'SilvercartProductVariantAttributeSet'      => 'SilvercartProductVariantAttributeSet',
        'SilvercartAttributedVariantAttributeSets'  => 'SilvercartAttributedVariantAttributeSet',
        'SilvercartShoppingCartPositions'           => 'SilvercartShoppingCartPosition'
    );
    
    /**
     * Returns the casted name
     *
     * @return string
     */
    public function getname() {
        return $this->getLanguageFieldValue('name');
    }

    /**
     * Returns the translated singular name of the object. If no translation exists
     * the class name will be returned.
     *
     * @return string The objects singular name
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 24.05.2012
     */
    public function singular_name() {
        return SilvercartTools::singular_name_for($this);
    }

    /**
     * Returns the translated plural name of the object. If no translation exists
     * the class name will be returned.
     *
     * @return string the objects plural name
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 24.05.2012
     */
    public function plural_name() {
        return SilvercartTools::plural_name_for($this);
    }
    
    /**
     * Field labels for display in tables.
     *
     * @param boolean $includerelations A boolean value to indicate if the labels returned include relation fields
     *
     * @return array
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 24.05.2012
     */
    public function fieldLabels($includerelations = true) {
        $fieldLabels = array_merge(
            parent::fieldLabels($includerelations),
            array(
                'name'                                          => _t('SilvercartProductVariantAttribute.NAME_LABEL'),
                'SilvercartProductVariantFieldModifiers'        => _t('SilvercartProductVariantFieldModifier.PLURALNAME'),
                'SilvercartProductVariantAttributeSet'          => _t('SilvercartProductVariantAttributeSet.PLURALNAME'),
                'SilvercartAttributedVariantAttributeSets'      => _t('SilvercartAttributedVariantAttributeSet.PLURALNAME'),
                'SilvercartShoppingCartPositions'               => _t('SilvercartShoppingCartPosition.PLURALNAME'),
                'SilvercartProductVariantAttributeLanguages'    => _t('SilvercartProductVariantAttributeLanguage.PLURALNAME'),
            )
        );

        $this->extend('updateFieldLabels', $fieldLabels);
        return $fieldLabels;
    }

    /**
     * Returns the searchable fields for this DataObject.
     *
     * @return array
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 19.12.2011
     */
    public function searchableFields() {
        $fields = parent::searchableFields();

        return $fields;
    }

    /**
     * Summaryfields for display in tables.
     *
     * @return array
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 24.05.2012
     */
    public function summaryFields() {
        $summaryFields = array(
            'name' => $this->fieldLabel('name'),
        );

        $this->extend('updateSummaryFields', $summaryFields);
        return $summaryFields;
    }

    /**
     * Returns the storeadmin GUI elements for this object.
     *
     * @param array $params Additional parameters
     *
     * @return FieldSet
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 24.05.2012
     */
    public function getCMSFields($params = null) {
        $fields = parent::getCMSFields($params);

        $fields->removeByName('SilvercartShoppingCartPositions');
        
        $languageFields = SilvercartLanguageHelper::prepareCMSFields($this->getLanguage(true));
        foreach ($languageFields as $languageField) {
            $fields->addFieldToTab('Root.Main', $languageField);
        }

        return $fields;
    }

    /**
     * Returns all cumulated field modifier notes for this attribute.
     *
     * @return string
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 29.11.2011
     */
    public function getFieldModifierNotes() {
        $notes = '';

        foreach ($this->SilvercartProductVariantFieldModifiers() as $fieldModifier) {
            if ($fieldModifier->shoppingCartNote != '') {
                if (!empty($notes)) {
                    $notes .= ', ';
                }
                $notes .= $fieldModifier->shoppingCartNote;
            }
        }

        return $notes;
    }

    /**
     * Returns the modified values (as influenced by the connected field
     * modifiers) for the given field.
     *
     * @param SilvercartProduct $silvercartProduct The SilvercartProduct object
     * @param string            $fieldName         The name of the field
     *
     * @return mixed
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 06.12.2011
     */
    public function getModifiedValueForField($silvercartProduct, $fieldName) {
        $fieldValue = $silvercartProduct->getField($fieldName);

        if ($this->SilvercartProductVariantFieldModifiers()) {
            foreach ($this->SilvercartProductVariantFieldModifiers() as $fieldModifier) {
                if ($fieldModifier->silvercartProductFieldName != $fieldName) {
                    continue;
                }

                $fieldValue = $fieldModifier->getResultForObject($silvercartProduct);
            }
        }

        return $fieldValue;
    }

    /**
     * We want to delete the relations to attribute sets and field modifiers
     * before deleting the attribute itself.
     *
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 18.01.2012
     */
    public function onBeforeDelete() {
        $fieldModifiers = array();

        foreach ($this->SilvercartProductVariantFieldModifiers() as $fieldModifier) {
            $fieldModifiers[] = $fieldModifier;
        }

        foreach ($fieldModifiers as $fieldModifier) {
            $fieldModifier->delete();
        }

        foreach ($this->SilvercartAttributedVariantAttributeSets() as $attributedAttributeSet) {
            $attributedAttributeSet->Attributes()->remove($this);
        }

        $attributeSets = DataObject::get('SilvercartProductVariantAttributeSet');

        if ($attributeSets) {
            foreach ($attributeSets as $attributeSet) {
                if ($attributeSet->Attributes()) {
                    $attributeSet->Attributes()->remove($this);
                }
            }
        }

        parent::onBeforeDelete();
    }
}
