<?php
/**
 * Copyright 2011 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package SilvercartProductVariants
 * @subpackage Base
 */

/**
 * Collects attributes into a set that can be attributed to a
 * SilvercartProductVariantWidget.
 *
 * @package SilvercartProductVariants
 * @subpackage Base
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @since 10.11.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @copyright 2011 pixeltricks GmbH
 */
class SilvercartProductVariantAttributeSet extends DataObject {

    /**
     * Attributes.
     *
     * @var array
     */
    public static $db = array(
        'type'              => "enum('attribute,userInput','attribute')",
        'dataSourceClass'   => 'VarChar(255)',
        'mustBeFilledIn'    => 'Boolean(0)',
        'SortOrder'         => 'Int'
    );

    /**
     * Has-many relationships.
     *
     * @var array
     */
    public static $has_many = array(
        'SilvercartProductVariantAttributeSetLanguages'    => 'SilvercartProductVariantAttributeSetLanguage',
    );

    /**
     * Many-many relationships.
     *
     * @var array
     */
    public static $many_many = array(
        'Attributes' => 'SilvercartProductVariantAttribute'
    );

    /**
     * Extrafields for relations.
     * 
     * @var array
     */
    public static $many_many_extraFields = array(
        'Attributes' => array(
            'SortOrder' => 'Int'
        )
    );

    /**
     * Castings.
     *
     * @var array
     */
    public static $casting = array(
        'name' => 'VarChar(255)'
    );
    
    /**
     * Returns the casted name
     *
     * @return string
     */
    public function getname() {
        return $this->getLanguageFieldValue('name');
    }

    /**
     * Returns the translated singular name of the object. If no translation exists
     * the class name will be returned.
     *
     * @return string The objects singular name
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 24.05.2012
     */
    public function singular_name() {
        return SilvercartTools::singular_name_for($this);
    }

    /**
     * Returns the translated plural name of the object. If no translation exists
     * the class name will be returned.
     *
     * @return string the objects plural name
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 24.05.2012
     */
    public function plural_name() {
        return SilvercartTools::plural_name_for($this);
    }

    /**
     * Returns a sorted list of attributes for this set.
     *
     * @return DataObjectSet
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 11.03.2012
     */
    public function getAttributes() {
        $attributes = $this->Attributes();

        $attributes->sort('SortOrder ASC');

        return $attributes;
    }

    /**
     * Returns the storeadmin GUI elements for this object.
     *
     * @param array $params Additional parameters
     *
     * @return FieldSet
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 24.05.2012
     */
    public function getCMSFields($params = null) {
        $insertAfterField   = 'type';
        $fields             = new FieldSet();
        $rootTabSet         = new TabSet('SilvercartProductVariantAttributeSet');
        $basicTab           = new Tab('basic', $this->fieldLabel('BasicTabName'));
        $translationTab     = new Tab('SilvercartProductVariantAttributeSetLanguagesTab', $this->fieldLabel('SilvercartProductVariantAttributeSetLanguages'));

        $fields->push($rootTabSet);

        if (empty($this->ID)) {
            $attributeTypesConditioned = array();
            $attributeTypes            = singleton('SilvercartProductVariantAttributeSet')->dbObject('type')->enumValues();

            foreach ($attributeTypes as $attributeType) {
                $attributeTypesConditioned[$attributeType] = _t('SilvercartProductVariantAttributeSet.TYPE_'.$attributeType);
            }

            // For creation of a DataObject
            $basicTab->push(
                new DropdownField(
                    'type',
                    $this->fieldLabel('type'),
                    $attributeTypesConditioned
                )
            );

            $rootTabSet->push($basicTab);
        } else {
            // For editing an existing DataObject
            $insertAfterField = 'SilvercartProductVariantAttributeSetTypeInfo';
            $basicTab->push(
                new LiteralField(
                    'SilvercartProductVariantAttributeSetTypeInfo',
                    sprintf(
                        '<h2>%s</h2><p><strong>%s</strong></p>',
                        $this->fieldLabel('TypeHeadline'),
                        _t('SilvercartProductVariantAttributeSet.TYPE_'.$this->type)
                    )
                )
            );
            $basicTab->push(
                new TextField(
                    'SortOrder',
                    $this->fieldLabel('SortOrder')
                )
            );

            $rootTabSet->push($basicTab);
            
            $translationTab->push(
                new ComplexTableField(
                    $this,
                    'SilvercartProductVariantAttributeSetLanguages',
                    'SilvercartProductVariantAttributeSetLanguage'
                )
            );
            $rootTabSet->push($translationTab);

            if ($this->type == 'attribute') {
                $this->getCMSFields_forAttribute($rootTabSet, $basicTab);
            } else {
                $this->getCMSFields_forUserInput($rootTabSet, $basicTab);
            }
        }
        
        $languageFields = SilvercartLanguageHelper::prepareCMSFields($this->getLanguage(true));
        foreach ($languageFields as $languageField) {
            $fields->insertAfter($languageField, $insertAfterField);
        }

        $this->extend('updateCMSFields', $fields);
        return $fields;
    }

    /**
     * Inserts attribute specific GUI elements.
     *
     * @param TabSet &$rootTabSet The root tab set
     * @param Tab    &$basicTab   The basic tab
     *
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 24.05.2012
     */
    public function getCMSFields_forAttribute(&$rootTabSet, &$basicTab) {

        // Basic tab
        $mustBeFilledInField = new CheckboxField('mustBeFilledIn', $this->fieldLabel('mustBeFilledIn'));
        $variantActionsField = new SilvercartVariantActionsField(
            $this,
            'variantActions',
            $this->fieldLabel('ActionsForSet')
        );
        $basicTab->push($mustBeFilledInField);
        $basicTab->push($variantActionsField);

        // Attributed attributes tab
        $attributedTab = new Tab('attributes', $this->fieldLabel('AttributesTabName'));
        $attributedField = new SilvercartManyManyOrderField(
            $this,
            'Attributes',
            'AttributesField',
            'Merkmale verwalten'
        );
        $attributedField->setRelationEditLink('admin/silvercart-product-variant-attribute/SilvercartProductVariantAttribute/');
        $attributedField->setRelationEditTabToOpen('Root_Main');
        $attributedTab->push($attributedField);

        $rootTabSet->insertAfter($attributedTab, $basicTab->name);

        $this->extend('updateCMSFields_forAttribute', $rootTabSet, $basicTab);
    }

    /**
     * Inserts product specific GUI elements.
     *
     * @param TabSet &$rootTabSet The root tab set
     * @param Tab    &$basicTab   The basic tab
     *
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 10.11.2011
     */
    public function getCMSFields_forUserInput(&$rootTabSet, &$basicTab) {
        $dataSourceClassField  = new TextField('dataSourceClass',  $this->fieldLabel('dataSourceClass'));

        $basicTab->push(
            $dataSourceClassField
        );

        $this->extend('updateCMSFields_forUserInput', $rootTabSet, $basicTab);
    }

    /**
     * Summaryfields for display in tables.
     *
     * @return array
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 24.05.2012
     */
    public function summaryFields() {
        $summaryFields = array(
            'name'              => $this->fieldLabel('name'),
            'dataSourceClass'   => $this->fieldLabel('dataSourceClass'),
        );

        $this->extend('updateSummaryFields', $summaryFields);
        return $summaryFields;
    }

    /**
     * Field labels for the storeadmin GUI
     *
     * @param boolean $includerelations a boolean value to indicate if the labels returned include relation fields
     *
     * @return array
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 24.05.2012
     */
    public function fieldLabels($includerelations = true) {
        $fieldLabels = parent::fieldLabels($includerelations);

        $fieldLabels = array_merge(
            $fieldLabels,
            array(
                'type'                                          => _t('SilvercartProductVariantAttributeSet.TYPE_LABEL'),
                'name'                                          => _t('SilvercartProductVariantAttributeSet.NAME_LABEL'),
                'dataSourceClass'                               => _t('SilvercartProductVariantAttributeSet.DATASOURCECLASS_LABEL'),
                'mustBeFilledIn'                                => _t('SilvercartProductVariantAttributeSet.MUSTBEFILLEDIN_LABEL'),
                'SortOrder'                                     => _t('SilvercartProductVariantAttributeSet.SORT_ORDER'),
                'SilvercartProductVariantAttributeSetLanguages' => _t('SilvercartProductVariantAttributeSetLanguage.PLURALNAME'),
                'BasicTabName'                                  => _t('SilvercartProductVariantAttributeSet.CMS_BASICTABNAME'),
                'TypeHeadline'                                  => _t('SilvercartProductVariantAttributeSet.TYPE_HEADLINE'),
                'ActionsForSet'                                 => _t('SilvercartProductVariantAttributeSet.ACTIONS_FOR_SET'),
                'AttributesTabName'                             => _t('SilvercartProductVariantAttributeSet.CMS_ATTRIBUTESTABNAME'),
                
            )
        );

        return $fieldLabels;
    }
}
