<?php
/**
 * Copyright 2011 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package SilvercartProductVariants
 * @subpackage Base
 */

/**
 * Defines a rule on how to modifiy a field.
 *
 * @package SilvercartProductVariants
 * @subpackage Base
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @since 23.11.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @copyright 2011 pixeltricks GmbH
 */
class SilvercartProductVariantFieldModifier extends DataObject {
    
    /**
     * Attributes.
     *
     * @var array
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 23.11.2011
     */
    public static $db = array(
        'silvercartProductFieldName'    => 'VarChar(255)',
        'modifierAction'                => "enum('add,subtract,multiply,divide,setTo','add')",
        'modifierValue'                 => 'VarChar(255)',
        'shoppingCartNote'              => 'VarChar(255)'
    );
    
    /**
     * Has-one relationships.
     *
     * @var array
     */
    public static $has_one = array(
        'SilvercartProductVariantAttribute'         => 'SilvercartProductVariantAttribute',
        'SilvercartAttributedVariantAttributeSet'   => 'SilvercartAttributedVariantAttributeSet',
    );

    /**
     * The default SilvercartProduct field name
     * 
     * @var string
     */
    public static $defaultFieldName = 'PriceGrossAmount';

    /**
     * The default field modifier action
     * 
     * @return string
     */
    public static $defaultFieldModifierAction= 'add';

    /**
     * The default field modifier value
     * 
     * @var string
     */
    public static $defaultFieldModifierValue= '';

    /**
     * Returns the default SilvercartProduct field to use for a field modifier.
     * 
     * @return string
     */
    public static function getDefaultFieldName() {
        return self::$defaultFieldName;
    }

    /**
     * Returns the default field modifier action.
     * 
     * @return string
     */
    public static function getDefaultFieldModifierAction() {
        return self::$defaultFieldModifierAction;
    }

    /**
     * Returns the default field modifier value.
     * 
     * @return string
     */
    public static function getDefaultFieldModifierValue() {
        return self::$defaultFieldModifierValue;
    }

    /**
     * Sets the default SilvercartProduct field to use for a field modifier.
     *
     * @param string $defaultFieldName The default field name
     *
     * @return void
     */
    public static function setDefaultFieldName($defaultFieldName) {
        self::$defaultFieldName = $defaultFieldName;
    }

    /**
     * Sets the default field modifier action.
     *
     * @param string $modifierAction The default modifier action to set
     *
     * @return void
     */
    public static function setDefaultFieldModifierAction($modifierAction) {
        self::$defaultFieldModifierAction = $modifierAction;
    }

    /**
     * Sets the default field modifier value.
     *
     * @param string $modifierValue The default modifier value to set
     *
     * @return void
     */
    public static function setDefaultFieldModifierValue($modifierValue) {
        self::$defaultFieldModifierValue = $modifierValue;
    }

    /**
     * Returns a list of black-listed fields that shouldn't be shown for
     * field modifiers.
     * 
     * @return array
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 18.01.2012
     */
    public function blackListFields() {
        $blackListFields = array(
            'ID',
            'ClassName',
            'Created',
            'EANCode',
            'LastEdited',
            'PriceGrossCurrency',
            'PriceNetCurrency',
            'PurchasePriceAmount',
            'PurchasePriceCurrency',
            'ImageID',
            'isFreeOfCharge',
            'MetaDescription',
            'MetaTitle',
            'MetaKeywords',
            'MSRPriceAmount',
            'MSRPriceCurrency',
            'ShortDescription',
            'LongDescription',
            'PurchaseMinDuration',
            'PurchaseMaxDuration',
            'PurchaseTimeUnit',
            'ProductNumberShop',
            'ProductNumberManufacturer',
            'isActive',
            'StockQuantity',
            'StockQuantityOverbookable',
            'PackagingQuantity',
            'SilvercartAvailabilityStatusID',
            'SilvercartTaxID',
            'SilvercartProductConditionID',
            'SilvercartProductGroupID',
            'SilvercartQuantityUnitID',
            'SilvercartManufacturerID',
            'SilvercartMasterProductID',
            'SilvercartNaturalRebateVoucherID',
            'SortOrder',
            'Weight'
        );

        $this->extend('updateBlackListFields', $blackListFields);

        return $blackListFields;
    }

    /**
     * Returns the translated singular name of the object. If no translation exists
     * the class name will be returned.
     * 
     * @return string The objects singular name 
     * 
     * @author Roland Lehmann <rlehmann@pixeltricks.de>
     * @since 14.07.2012
     */
    public function singular_name() {
        return SilvercartTools::singular_name_for($this);
    }


    /**
     * Returns the translated plural name of the object. If no translation exists
     * the class name will be returned.
     * 
     * @return string the objects plural name
     * 
     * @author Roland Lehmann <rlehmann@pixeltricks.de>
     * @since 14.07.2012
     */
    public function plural_name() {
        return SilvercartTools::plural_name_for($this); 
    }
    
    /**
     * Field labels for display in tables.
     *
     * @param boolean $includerelations A boolean value to indicate if the labels returned include relation fields
     *
     * @return array
     *
     * @author Roland Lehmann <rlehmann@pixeltricks.de>
     * @copyright 2012 pixeltricks GmbH
     * @since 14.07.2012
     */
    public function fieldLabels($includerelations = true) {
        $fieldLabels = array_merge(
                parent::fieldLabels($includerelations),             array(
                    'silvercartProductFieldName'    => _t('SilvercartProductVariantFieldModifier.FIELDNAME_LABEL'),
                    'modifierAction'                => _t('SilvercartProductVariantFieldModifier.MODIFYACTIONFIELD_LABEL'),
                    'modifierValue'                 => _t('SilvercartProductVariantFieldModifier.MODIFYVALUEFIELD_LABEL'),
                    'TabBasic'                      => _t('SilvercartProductVariantFieldModifier.TAB_BASIC_SETTINGS'),
                    'ShoppingCartNote'              => _t('SilvercartProductVariantFieldModifier.SHOPPINGCARTNOTE_LABEL'),
                    'ActionAdd'                     => _t('SilvercartProductVariantFieldModifier.ACTION_ADD'),
                    'ActionAddBy'                   => _t('SilvercartProductVariantFieldModifier.ACTION_ADD_BY'),
                    'ActionSubtract'                => _t('SilvercartProductVariantFieldModifier.ACTION_SUBTRACT'),
                    'ActionSubtractBy'              => _t('SilvercartProductVariantFieldModifier.ACTION_SUBTRACT_BY'),
                    'ActionMultiply'                => _t('SilvercartProductVariantFieldModifier.ACTION_MULTIPLY'),
                    'ActionMultiplyBy'              => _t('SilvercartProductVariantFieldModifier.ACTION_MULTIPLY_BY'),
                    'ActionDivide'                  => _t('SilvercartProductVariantFieldModifier.ACTION_DIVIDE'),
                    'ActionDivideBy'                => _t('SilvercartProductVariantFieldModifier.ACTION_DIVIDE_BY'),
                    'ActionSetTo'                   => _t('SilvercartProductVariantFieldModifier.ACTION_SETTO'),
                    
                )
        );

        $this->extend('updateFieldLabels', $fieldLabels);
        return $fieldLabels;
    }
    
    /**
     * Summaryfields for display in tables.
     *
     * @return array
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 23.11.2011
     */
    public function summaryFields() {
        $summaryFields = array(
            'silvercartProductFieldName'    => $this->fieldLabel('silvercartProductFieldName'),
            'modifierAction'                => $this->fieldLabel('modifierAction'),
            'modifierValue'                 => $this->fieldLabel('modifierValue'),
            
            );
        
        $this->extend('updateSummaryFields', $summaryFields);
        return $summaryFields;
    }
    
    /**
     * Returns the storeadmin GUI elements for this object.
     * 
     * @param array $params Additional parameters
     * 
     * @return FieldSet
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 23.11.2011
     */
    public function getCMSFields($params = null) {
        $tabset     = new TabSet('Sections');
        
        $tabBasic = new Tab('Basic', $this->fieldLabel('TabBasic'));
        $tabset->push($tabBasic);
        
        $availableFields           = $this->getProductFieldList();
        $availableFieldsTranslated = array();

        foreach ($availableFields as $availableField) {
            $availableFieldsTranslated[$availableField->fieldName] = $availableField->fieldTitle;
        }

        $fieldActions           = $this->getModifierActions();
        $fieldActionsTranslated = array();

        foreach ($fieldActions as $fieldAction) {
            $fieldActionsTranslated[$fieldAction->action] = $fieldAction->actionTitle;
        }

        // Define fields
        $productFieldDropdownField = new DropdownField(
            'silvercartProductFieldName',
            $this->fieldLabel('silvercartProductFieldName'),
            $availableFieldsTranslated
        );
        $modifyActionField = new DropdownField(
            'modifierAction',
            $this->fieldLabel('modifierAction'),
            $fieldActionsTranslated
        );
        $modifyValueField = new TextField(
            'modifierValue',
            $this->fieldLabel('modifierValue')
        );
        $shoppingCartNoteField = new TextField(
            'shoppingCartNote',
            $this->fieldLabel('ShoppingCartNote')
        );
        
        $tabBasic->setChildren(
            new FieldSet(
                $productFieldDropdownField,
                $modifyActionField,
                $modifyValueField,
                $shoppingCartNoteField
            )
        );
        
        return new FieldSet($tabset);
    }

    /**
     * Returns the associated attribute ID.
     * 
     * @return int
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 18.01.2012
     */
    public function getAttributeID() {
        return $this->SilvercartProductVariantAttribute()->ID;
    }

    /**
     * Returns all fields for a SilvercartProduct as DataObjectSet.
     * 
     * @return DataObjectSet
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 18.01.2012
     */
    public function getProductFieldList() {
        $dbFields        = DataObject::database_fields('SilvercartProduct');
        $availableFields = array();
        $fieldLabels     = singleton('SilvercartProduct')->fieldLabels();
        
        foreach ($dbFields as $fieldName => $fieldType) {

            if (array_key_exists($fieldName, $fieldLabels)) {
                $fieldLabel = $fieldLabels[$fieldName];
            } else {
                $fieldLabel = $fieldName;
            }

            if ($this->silvercartProductFieldName == $fieldName) {
                $isSelected = true;
            } else {
                $isSelected = false;
            }

            if (!in_array($fieldName, $this->blackListFields())) {
                $availableFields[$fieldName] = new DataObject(
                    array(
                        'fieldName'  => $fieldName,
                        'fieldTitle' => $fieldLabel,
                        'isSelected' => $isSelected
                    )
                );
            }
        }

        return new DataObjectSet($availableFields);
    }

    /**
     * Returns all modifier actions as DataObjectSet
     * 
     * @return DataObjectSet
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 18.01.2012
     */
    public function getModifierActions() {
        $actions = new DataObjectSet(
            array(
                'add' => new DataObject(
                    array(
                        'action'      => 'add',
                        'name'        => $this->fieldLabel('ActionAdd'),
                        'actionTitle' => $this->fieldLabel('ActionAddBy'),
                        'isSelected'  => $this->modifierAction == 'add' ? true : false
                    )
                ),
                'subtract' => new DataObject(
                    array(
                        'action'      => 'subtract',
                        'name'        => $this->fieldLabel('ActionSubtract'),
                        'actionTitle' => $this->fieldLabel('ActionSubtractBy'),
                        'isSelected'  => $this->modifierAction == 'subtract' ? true : false
                    )
                ),
                'multiply' => new DataObject(
                    array(
                        'action'      => 'multiply',
                        'name'        => $this->fieldLabel('ActionMultiply'),
                        'actionTitle' => $this->fieldLabel('ActionMultiplyBy'),
                        'isSelected'  => $this->modifierAction == 'multiply' ? true : false
                    )
                ),
                'divide' => new DataObject(
                    array(
                        'action'      => 'divide',
                        'name'        => $this->fieldLabel('ActionDivide'),
                        'actionTitle' => $this->fieldLabel('ActionDivideBy'),
                        'isSelected'  => $this->modifierAction == 'divide' ? true : false
                    )
                ),
                'setTo' => new DataObject(
                    array(
                        'action'      => 'setTo',
                        'name'        => $this->fieldLabel('ActionSetTo'),
                        'actionTitle' => $this->fieldLabel('ActionSetTo'),
                        'isSelected'  => $this->modifierAction == 'setTo' ? true : false
                    )
                )
            )
        );

        return $actions;
    }
    
    /**
     * Applies the modifier to the given SilvercartProduct instance.
     *
     * @param SilvercartProduct $silvercartProduct The object to apply the modifier to
     * @param string            $getValueFunction  An optional function name that should
     *                                             be used to retrieve the value
     * @param string            $valueFunction     An optional function name that should
     *                                             be used to get value from a returned object.
     *
     * @return mixed
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 23.1.2011
     */
    public function getResultForObject($silvercartProduct, $getValueFunction = '', $valueFunction = '') {
        if (!$silvercartProduct->hasField($this->silvercartProductFieldName)) {
            return false;
        }

        if (!empty($getValueFunction) &&
            !empty($valueFunction)) {

            $originalValue = $silvercartProduct->$getValueFunction()->$valueFunction();
        } else if (!empty($getValueFunction)) {
            $originalValue = $silvercartProduct->$getValueFunction();
        } else {
            $originalValue = $silvercartProduct->getField($this->silvercartProductFieldName);
        }

        $calculatedValue = null;
        
        switch ($this->modifierAction) {
            case 'add':
                $calculatedValue = $originalValue + $this->modifierValue;
                break;
            case 'subtract':
                $calculatedValue = $originalValue - $this->modifierValue;
                break;
            case 'multiply':
                $calculatedValue = $originalValue * $this->modifierValue;
                break;
            case 'divide':
                if (!empty($this->modifierValue) &&
                     $this->modifierValue != 0) {
                    
                    $calculatedValue = $originalValue / $this->modifierValue;
                }
                break;
            case 'setTo':
                $calculatedValue = $this->modifierValue;
                break;
        }

        return $calculatedValue;
    }
}
