<?php
/**
 * Copyright 2011 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package SilvercartProductVariants
 * @subpackage Base
 */

/**
 * Contains configuration options for product variants.
 *
 * @package SilvercartProductVariants
 * @subpackage Base
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @since 16.11.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @copyright 2011 pixeltricks GmbH
 */
class SilvercartProductVariantConfiguration extends DataObject {
    
    /**
     * Attributes.
     *
     * @var array
     */
    public static $db = array(
        'name'                                  => 'VarChar(255)',
        'hideDeactivatedAttributes'             => 'Boolean(0)',
        'fieldModifiersForAssignedAttributes'   => 'Boolean(0)',
    );

    /**
     * Defaults for empty fields.
     *
     * @var array
     */
    public static $defaults = array(
        'fieldModifiersForAssignedAttributes' => true
    );
    
    /**
     * The configuration fields should have a static attribute to set after its
     * first call (to prevent redundant logic).
     * 
     * @var boolean
     */
    public static $hideDeactivatedAttributes            = null;
    public static $config                               = null;
    public static $fieldModifiersForAssignedAttributes  = null;
    
    /**
     * Sets the translations of SilvercartConfigs field labels.
     *
     * @param bool $includerelations Include relations or not
     *
     * @return array
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 16.11.2011
     */
    public function fieldLabels($includerelations = true) {
        $fieldLabels = parent::fieldLabels($includerelations);
        $fieldLabels['name']                                = _t('SilvercartProductVariantConfig.NAME_LABEL');
        $fieldLabels['hideDeactivatedAttributes']           = _t('SilvercartProductVariantConfig.HIDE_DEACTIVATED_ATTRIBUTES');
        $fieldLabels['fieldModifiersForAssignedAttributes'] = _t('SilvercartProductVariantConfig.FIELDMODIFIERS_FOR_ASSIGNED_ATTRIBUTES');

        return $fieldLabels;
    }
    
    /**
     * Summaryfields for display in tables.
     *
     * @return array
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 10.07.2012
     */
    public function summaryFields() {
        $summaryFields = array(
            'name' => $this->fieldLabel('name'),
        );
        
        $this->extend('updateSummaryFields', $summaryFields);
        return $summaryFields;
    }
    
    /**
     * Returns the translated singular name of the object. If no translation exists
     * the class name will be returned.
     * 
     * @return string The objects singular name 
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 10.07.2012
     */
    public function singular_name() {
        return SilvercartTools::singular_name_for($this);
    }
    
    /**
     * Returns the translated plural name of the object. If no translation exists
     * the class name will be returned.
     * 
     * @return string the objects plural name
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 10.07.2012
     */
    public function plural_name() {
        return SilvercartTools::plural_name_for($this);
    }

    /**
     * Returns whether deactivated attributes should be hidden.
     *
     * @return bool
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 08.03.2012
     */
    public static function HideDeactivatedAttributes() {
        if (is_null(self::$hideDeactivatedAttributes)) {
            self::$hideDeactivatedAttributes = self::getConfig()->hideDeactivatedAttributes;
        }
        return self::$hideDeactivatedAttributes;
    }

    /**
     * Returns whether field modifier rules should apply to assigned
     * attributes only.
     *
     * @return bool
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 2013-02-06
     */
    public static function FieldModifiersForAssignedAttributes() {
        if (is_null(self::$fieldModifiersForAssignedAttributes)) {
            self::$fieldModifiersForAssignedAttributes = self::getConfig()->fieldModifiersForAssignedAttributes;
        }
        return self::$fieldModifiersForAssignedAttributes;
    }
    
    /**
     * Returns the SilvercartConfig or triggers an error if not existent.
     *
     * @return SilvercartConfig
     */
    public static function getConfig() {
        if (is_null(self::$config)) {
            self::$config = DataObject::get_one('SilvercartProductVariantConfiguration');
            if (!self::$config) {
                $errorMessage = _t('SilvercartProductVariantConfig.ERROR_NO_CONFIG');
                self::triggerError($errorMessage);
            }
        }
        return self::$config;
    }
}
