<?php
/**
 * Copyright 2011 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package SilvercartProductVariants
 * @subpackage FormFields
 */

/**
 * A formfield that displays a list of product variants and provides buttons
 * to easily add and remove variants.
 *
 * @package SilvercartProductVariants
 * @subpackage FormFields
 * @copyright pixeltricks GmbH
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @since 10.11.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 */
class SilvercartVariantTableListField extends FormField {

    /**
     * The controller object.
     *
     * @var Object
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 11.10.2011
     */
    protected $controller;

    /**
     * Constructor
     *
     * @param Controller $controller The controller object.
     * @param string     $name       The internal field name, passed to forms.
	 * @param string     $title      The field label.
     *
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 10.11.2011
     */
    public function __construct($controller, $name, $title = null) {
        parent::__construct($name, $title);

        $this->controller = $controller;
    }

    /**
     * Returns the rendered HTML code for this field.
     *
     * @return string
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 10.11.2011
     */
    public function FieldHolder() {
        $cssFile = 'silvercart_product_variants/css/SilvercartVariantTableListField.css';
        Requirements::css($cssFile);

        $data = self::getTableFieldData($this->controller);

        $pluginData = SilvercartPlugin::call(
            $this,
            'FieldHolder',
            array(
                $data
            ),
            true,
            array()
        );

        if (is_array($pluginData) &&
            $pluginData[0] !== false) {

            $data = $pluginData[0];
        }

        return $this->customise($data)->renderWith($this->class);
    }

    /**
     * Returns the SilvercartVariantTableListField for ajax requests as HTML
     * code.
     *
     * @param SilvercartProduct $product              Product to get field for
     * @param int               $activeAttributeSetId The ID of the active attribute set
     *
     * @return string
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 24.05.2012
     */
    public static function getTableFieldData($product, $activeAttributeSetId = null) {
        $assignedAttributeSets   = array();
        $unAssignedAttributeSets = array();
        $attributeSets           = DataObject::get('SilvercartProductVariantAttributeSet');
        $attributedSets          = new DataObjectSet();
        $fieldModifierCount      = 0;

        // Get assigned and unassigned attribute sets
        if ($attributeSets) {
            foreach ($attributeSets as $attributeSet) {
                if ($product->SilvercartProductVariantAttributeSets()->find('ID', $attributeSet->ID)) {
                    $assignedAttributeSets[] = $attributeSet;
                } else {
                    $unAssignedAttributeSets[] = $attributeSet;
                }
            }
        }
        
        $current_locale             = Translatable::get_current_locale();
        $silvercartDefaultLocale    = SilvercartConfig::Locale();

        if (SilvercartConfig::useDefaultLanguageAsFallback() &&
            $current_locale != $silvercartDefaultLocale &&
            !empty ($silvercartDefaultLocale)) {
            $where = sprintf(
                "`AL`.`Locale` = IFNULL((%s), (%s))",
                self::getLocaleDependantSelect($current_locale),
                self::getLocaleDependantSelect($silvercartDefaultLocale)
            );
        } elseif (!empty ($silvercartDefaultLocale)) {
            $where = sprintf(
                "`AL`.`Locale` = '%s'",
                $current_locale
            );
        }

        foreach ($product->SilvercartAttributedVariantAttributeSets() as $attributeSet) {
            $setAttributes = new DataObjectSet();
            $records       = DB::query(
                sprintf(
                    "
                    SELECT
                        AVAS_A.ID,
                        AVAS_A.isActive,
                        AVAS_A.isDefault,
                        AL.name,
                        A.ID AS SilvercartProductVariantAttributeID
                    FROM
                        SilvercartAttributedVariantAttributeSet_Attributes AVAS_A
                    LEFT JOIN
                        SilvercartProductVariantAttribute A
                    ON
                        A.ID = AVAS_A.SilvercartProductVariantAttributeID
                    LEFT JOIN
                        SilvercartProductVariantAttributeLanguage AL
                    ON
                        A.ID = AL.SilvercartProductVariantAttributeID
                    WHERE
                        AVAS_A.SilvercartAttributedVariantAttributeSetID = %d
                    AND
                        %s
                    ORDER BY
                        AVAS_A.SilvercartAttributedVariantAttributeSetID ASC,
                        AL.name ASC
                    ",
                    $attributeSet->ID,
                    $where
                )
            );

            if ($records) {
                foreach ($records as $record) {
                    if (!$record['isActive'] &&
                         SilvercartProductVariantConfiguration::HideDeactivatedAttributes()) {

                         continue;
                    }

                    $attribute              = DataObject::get_by_id('SilvercartProductVariantAttribute', $record['SilvercartProductVariantAttributeID']);
                    $filteredFieldModifiers = new DataObjectSet();
                    $hasFieldModifiers      = false;

                    if ($attribute &&
                        $attribute->SilvercartProductVariantFieldModifiers()->Count() > 0) {
                        $fieldModifiers = $attribute->SilvercartProductVariantFieldModifiers();

                        if ($fieldModifiers) {
                            foreach ($fieldModifiers as $fieldModifier) {
                                if (SilvercartProductVariantConfiguration::FieldModifiersForAssignedAttributes()) {
                                    if ($fieldModifier->SilvercartAttributedVariantAttributeSetID == $attributeSet->ID) {
                                        $fieldModifier->fieldModifierCount     = $fieldModifierCount;
                                        $fieldModifier->attributedVariantSetId = $attributeSet->ID;

                                        $filteredFieldModifiers->push($fieldModifier);
                                        $fieldModifierCount++;
                                    }
                                } else {
                                    $fieldModifier->fieldModifierCount     = $fieldModifierCount;
                                    $fieldModifier->attributedVariantSetId = $attributeSet->ID;

                                    $filteredFieldModifiers->push($fieldModifier);
                                    $fieldModifierCount++;
                                }
                            }
                        }

                        if ($fieldModifierCount > 0) {
                            $hasFieldModifiers = true;
                        }
                    }

                    if ($attribute) {
                        $setAttributes->push(
                            new DataObject(
                                array(
                                    'ID'                                        => $record['ID'],
                                    'name'                                      => $record['name'],
                                    'isActive'                                  => $record['isActive'],
                                    'isDefault'                                 => $record['isDefault'],
                                    'SilvercartProductVariantFieldModifiers'    => $filteredFieldModifiers,
                                    'hasFieldModifiers'                         => $hasFieldModifiers
                                )
                            )
                        );
                    }
                }
            }
            $attributedSets->push(
                new DataObject(
                    array(
                        'name'                                  => $attributeSet->SilvercartProductVariantAttributeSet()->name,
                        'SilvercartProductVariantAttributeSet'  => $attributeSet,
                        'Attributes'                            => $setAttributes,
                        'AttributedSet'                         => $attributeSet
                    )
                )
            );
        }

        $data = array(
            'SilvercartAttributedVariantAttributeSets'  => $attributedSets,
            'assignedAttributeSets'                     => new DataObjectSet($assignedAttributeSets),
            'unAssignedAttributeSets'                   => new DataObjectSet($unAssignedAttributeSets),
            'hideDeactivatedAttributes'                 => SilvercartProductVariantConfiguration::HideDeactivatedAttributes(),
            'setActiveAttributeSetId'                   => $activeAttributeSetId === null ? false : true,
            'activeAttributeSetId'                      => $activeAttributeSetId
        );

        return $data;
    }
    
    /**
     * Returns a locale dependant select statement
     *
     * @param string $locale Locale to get statement for
     * 
     * @return string
     */
    public static function getLocaleDependantSelect($locale) {
        $localeDependantSelect = sprintf(
                "SELECT `SPVAL`.`Locale` FROM `SilvercartProductVariantAttributeLanguage` AS `SPVAL` WHERE `SPVAL`.`Locale` = '%s' AND `A`.`ID` = `SPVAL`.`SilvercartProductVariantAttributeID` LIMIT 0,1",
                $locale
        );
        return $localeDependantSelect;
    }
}
