<?php
/**
 * Copyright 2011 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package SilvercartProductVariants
 * @subpackage Order
 */

/**
 * Attaches new attributes.
 *
 * @package SilvercartProductVariants
 * @subpackage Order
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @since 16.11.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @copyright 2011 pixeltricks GmbH
 */
class SilvercartProductVariantShoppingCartPosition extends DataObjectDecorator {
    
    /**
     * Extends the database fields and relations of the decorated class.
     *
     * @return array
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 16.11.2011
     */
    public function extraStatics() {
        return array(
            'many_many' => array(
                'ProductVariantAttributes'   => 'SilvercartProductVariantAttribute',
                'ProductVariantUGAttributes' => 'SilvercartProductVariantUserGeneratedAttribute'
            ),
            'many_many_extraFields' => array(
                'ProductVariantAttributes' => array(
                    'SilvercartProductVariantAttributeSet' => 'Int'
                )
            )
        );
    }
    
    /**
     * Returns a DataObjectSet for iteration in templates containing the
     * attributed SilvercartProductVariantAttributes.
     *
     * @return DataObjectSet
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 17.11.2011
     */
    public function getAttributedVariantAttributes() {
        $attributeSets = new DataObjectSet;
        
        // --------------------------------------------------------------------
        // Get standard attributes
        // --------------------------------------------------------------------
        foreach ($this->owner->ProductVariantAttributes() as $attribute) {
            
            $silvercartProductVariantAttributeSet = DataObject::get_by_id('SilvercartProductVariantAttributeSet', $attribute->SilvercartProductVariantAttributeSet);
            
            if ($silvercartProductVariantAttributeSet) {
                $setName            = $silvercartProductVariantAttributeSet->name;
                $fieldModifierNotes = $attribute->getFieldModifierNotes();
                
                if ($fieldModifierNotes != '') {
                    $setName .= '<br />('.$fieldModifierNotes.')';
                }
                
                $attributeSets->push(
                    new DataObject(
                        array(
                            'name'      => $attribute->name,
                            'setName'   => $setName
                        )
                    )
                );
            }
        }
        
        // --------------------------------------------------------------------
        // Get user generated attributes
        // --------------------------------------------------------------------
        foreach ($this->owner->ProductVariantUGAttributes() as $ugAttribute) {
            $sourceClassName = $ugAttribute->inputObjectClassName;
            $sourceClassID   = $ugAttribute->inputObjectID;
            
            $sourceClassObj = DataObject::get_by_id($sourceClassName, $sourceClassID);
            
            if ($sourceClassObj) {
                $shoppingCartPositionInfo = $sourceClassObj->getShoppingCartInfo();
                
                if ($shoppingCartPositionInfo) {
                    $attributeSets->push($shoppingCartPositionInfo);
                }
            }
        }
        
        return $attributeSets;
    }
    
    /**
     * We delete the attributed ProductVariantAttributes here.
     *
     * @return void
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 17.11.2011
     */
    public function updateOnBeforeDelete() {
        if ($this->owner->ProductVariantAttributes()) {
            $this->owner->ProductVariantAttributes()->removeAll();
        }
        if ($this->owner->ProductVariantUGAttributes()) {
            foreach ($this->owner->ProductVariantUGAttributes() as $ugAttribute) {
                $ugAttribute->delete();
            }
            
            $this->owner->ProductVariantUGAttributes()->removeAll();
        }
    }

    /**
     * Transfer attributed variants to a new cart position.
     *
     * @param SilvercartShoppingCartPosition &$newShoppingCartPosition The new cart position
     *
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 14.03.2012
     */
    public function updateTransferToNewPosition(&$newShoppingCartPosition) {
        DB::query(
            sprintf(
                "
                UPDATE
                    SilvercartShoppingCartPosition_ProductVariantAttributes
                SET
                    SilvercartShoppingCartPositionID = %d
                WHERE
                    SilvercartShoppingCartPositionID = %d
                ",
                $newShoppingCartPosition->ID,
                $this->owner->ID
            )
        );
        DB::query(
            sprintf(
                "
                UPDATE
                    SilvercartShoppingCartPosition_ProductVariantUGAttributes
                SET
                    SilvercartShoppingCartPositionID = %d
                WHERE
                    SilvercartShoppingCartPositionID = %d
                ",
                $newShoppingCartPosition->ID,
                $this->owner->ID
            )
        );
    }
}
