<?php
/**
 * Copyright 2011 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package SilvercartProductVariants
 * @subpackage Plugins
 */

/**
 * Delivers additional functionality for SilvercartShoppingCartPosition.
 *
 * @package SilvercartProductVariants
 * @subpackage Plugins
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @since 23.11.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @copyright 2011 pixeltricks GmbH
 */
class SilvercartProductVariantShoppingCartPositionPlugin extends DataObjectDecorator {
    
    /**
     * Overwrites the getPrice method.
     *
     * @param array &$arguments     The arguments to pass
     *                              $arguments[0]: $forSingleProduct
     * @param mixed &$callingObject The calling object (SilvercartShoppingcartPosition)
     * 
     * @return string
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 25.06.2012
     */
    public function pluginOverwriteGetPrice(&$arguments, &$callingObject) {
        $product                       = $callingObject->SilvercartProduct();
        $price                         = 0;
        $priceGrossAddFromAttributes   = 0;
        $priceNetAddFromAttributes     = 0;
        $priceGrossAddFromUGAttributes = 0;
        $priceNetAddFromUGAttributes   = 0;
        $priceObj                      = null;

        if ($product) {
            $originalPrice = $product->getPrice()->getAmount();

            // ----------------------------------------------------------------
            // Get modifiers for standard attributes
            // ----------------------------------------------------------------
            foreach ($callingObject->ProductVariantAttributes() as $attribute) {
                foreach ($attribute->SilvercartProductVariantFieldModifiers() as $modifier) {
                    if ($modifier->silvercartProductFieldName == 'PriceGrossAmount') {
                        $price                        = $modifier->getResultForObject($product, 'getPrice', 'getAmount');
                        $priceGrossAddFromAttributes += $price - $originalPrice;
                    }
                    if ($modifier->silvercartProductFieldName == 'PriceNetAmount') {
                        $price                      = $modifier->getResultForObject($product, 'getPrice', 'getAmount');
                        $priceNetAddFromAttributes += $price - $originalPrice;
                    }
                }
            }

            // ----------------------------------------------------------------
            // Get modifiers for user generated attributes
            // ----------------------------------------------------------------
            foreach ($callingObject->ProductVariantUGAttributes() as $ugAttribute) {
                $sourceClassName = $ugAttribute->inputObjectClassName;
                $sourceClassID   = $ugAttribute->inputObjectID;
                $sourceClassObj  = DataObject::get_by_id($sourceClassName, $sourceClassID);
                
                if ($sourceClassObj) {
                    $fieldModifiers = $sourceClassObj->getFieldModifiers();
                    
                    if ($fieldModifiers) {
                        foreach ($fieldModifiers as $modifier) {
                            if ($modifier->silvercartProductFieldName == 'PriceGrossAmount') { 
                                $price                          = $modifier->getResultForObject($product, 'getPrice', 'getAmount');
                                $priceGrossAddFromUGAttributes += $price - $originalPrice;
                            }
                            if ($modifier->silvercartProductFieldName == 'PriceNetAmount') { 
                                $price                        = $modifier->getResultForObject($product, 'getPrice', 'getAmount');
                                $priceNetAddFromUGAttributes += $price - $originalPrice;
                            }
                        }
                    }
                }
            }
            $price = $originalPrice + $priceNetAddFromAttributes + $priceGrossAddFromAttributes + $priceNetAddFromUGAttributes + $priceGrossAddFromUGAttributes;

            if ($price != $originalPrice) {
                if (!$arguments[0]) {
                    $price = $price * $callingObject->Quantity;
                }

                $priceObj = new Money();
                $priceObj->setAmount($price);
                $priceObj->setCurrency(SilvercartConfig::DefaultCurrency());
            }
        }

        return $priceObj;
    }

    /**
     * Overwrites the getPrice method.
     *
     * @param array &$arguments     The arguments to pass
     *                              $arguments[0]: $forSingleProduct
     * @param mixed &$callingObject The calling object (SilvercartShoppingcartPosition)
     *
     * @return string
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 25.06.2012
     */
    public function pluginOverwriteGetProductNumberShop(&$arguments, &$callingObject) {
        $product       = $callingObject->SilvercartProduct();
        $productNumber = $product->ProductNumberShop;

        if ($product) {
            // ----------------------------------------------------------------
            // Get modifiers for standard attributes
            // ----------------------------------------------------------------
            if (SilvercartProductVariantConfiguration::FieldModifiersForAssignedAttributes()) {
                foreach ($callingObject->ProductVariantAttributes() as $attribute) {

                    $silvercartProductVariantAttributeSet = DataObject::get_by_id('SilvercartProductVariantAttributeSet', $attribute->SilvercartProductVariantAttributeSet);

                    if ($silvercartProductVariantAttributeSet) {

                        foreach ($product->SilvercartAttributedVariantAttributeSets() as $attributedSet) {
                            if ($attributedSet->SilvercartProductVariantAttributeSetID == $silvercartProductVariantAttributeSet->ID) {
                                foreach ($attribute->SilvercartProductVariantFieldModifiers() as $fieldModifier) {
                                    if ($fieldModifier->SilvercartAttributedVariantAttributeSetID == $attributedSet->ID) {
                                        if ($fieldModifier->silvercartProductFieldName == 'ProductNumberShop') {
                                            $productNumber = $fieldModifier->getResultForObject($product);
                                            break;
                                        }
                                    }
                                }
                                break;
                            }
                        }
                    }
                }
            } else {
                foreach ($callingObject->ProductVariantAttributes() as $attribute) {
                    foreach ($attribute->SilvercartProductVariantFieldModifiers() as $modifier) {
                        if ($modifier->silvercartProductFieldName == 'ProductNumberShop') {
                            $productNumber = $modifier->getResultForObject($product);
                            break;
                        }
                    }
                }
            }

            // ----------------------------------------------------------------
            // Get modifiers for user generated attributes
            // ----------------------------------------------------------------
            foreach ($callingObject->ProductVariantUGAttributes() as $ugAttribute) {
                $sourceClassName = $ugAttribute->inputObjectClassName;
                $sourceClassID   = $ugAttribute->inputObjectID;
                $sourceClassObj  = DataObject::get_by_id($sourceClassName, $sourceClassID);

                if ($sourceClassObj) {
                    $fieldModifiers = $sourceClassObj->getFieldModifiers();

                    if ($fieldModifiers) {
                        foreach ($fieldModifiers as $modifier) {
                            if ($modifier->silvercartProductFieldName == 'ProductNumberShop') {
                                $productNumber = $modifier->getResultForObject($product);
                            }
                        }
                    }
                }
            }
        }

        return $productNumber;
    }

    /**
     * Overwrites the getTitle method.
     *
     * @param array &$arguments     The arguments to pass
     * @param mixed &$callingObject The calling object (SilvercartShoppingcartPosition)
     * 
     * @return string
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 23.11.2011
     */
    public function pluginOverwriteGetTitle(&$arguments, &$callingObject) {
        $product = $callingObject->SilvercartProduct();
        $title   = '';

        if ($product && $product->getTitle()) {
            $originalTitle = $product->getTitle();

            // ----------------------------------------------------------------
            // Get modifiers for standard attributes
            // ----------------------------------------------------------------
            foreach ($callingObject->ProductVariantAttributes() as $attribute) {
                foreach ($attribute->SilvercartProductVariantFieldModifiers() as $modifier) {
                    if ($modifier->silvercartProductFieldName == 'Title') {
                        $title .= $modifier->getResultForObject($product);
                    }
                }
            }
            
            // ----------------------------------------------------------------
            // Get modifiers for user generated attributes
            // ----------------------------------------------------------------
            foreach ($callingObject->ProductVariantUGAttributes() as $ugAttribute) {
                $sourceClassName = $ugAttribute->inputObjectClassName;
                $sourceClassID   = $ugAttribute->inputObjectID;
                $sourceClassObj  = DataObject::get_by_id($sourceClassName, $sourceClassID);
                
                if ($sourceClassObj) {
                    $fieldModifiers = $sourceClassObj->getFieldModifiers();
                    
                    if ($fieldModifiers) {
                        foreach ($fieldModifiers as $modifier) {
                            if ($modifier->silvercartProductFieldName == 'Title') {
                                $title .= $modifier->getResultForObject($product);
                            }
                        }
                    }
                }
            }
        }

        return $title;
    }
    
    /**
     * Returns an extension for the shopping cart position title
     *
     * @param SilvercartShoppingCartPosition $position Position to add title extension
     * 
     * @return string
     * 
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 25.06.2012
     */
    public function pluginAddToTitle($position) {
        return $position->renderWith('SilvercartProductVariantShoppingCartPositionAddToTitle');
    }
    
    /**
     * Returns an extension for the shopping cart position title
     *
     * @param SilvercartShoppingCartPosition $position Position to add title extension
     * 
     * @return string
     * 
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 12.07.2012
     */
    public function pluginAddToTitleForWidget($position) {
        return $position->renderWith('SilvercartProductVariantShoppingCartPositionAddToTitleForWidget');
    }
}
