<?php
/**
 * Copyright 2011 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package SilvercartProductVariants
 * @subpackage Products
 */

/**
 * Extends the SilvercartProduct by variant specific attributes and methods.
 *
 * @package SilvercartProductVariants
 * @subpackage Products
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @since 10.11.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @copyright 2011 pixeltricks GmbH
 */
class SilvercartProductVariantProduct extends DataObjectDecorator {

    /**
     * Indicates whether this product has variants
     *
     * @var bool
     */
    protected $hasVariants = null;
    
    /**
     * Decorates attributes and relationships.
     *
     * @return array
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 10.11.2011
     */
    public function extraStatics() {
        return array(
            'has_many' => array(
                'SilvercartAttributedVariantAttributeSets' => 'SilvercartAttributedVariantAttributeSet'
            ),
            'many_many' => array(
                'SilvercartProductVariantAttributeSets' => 'SilvercartProductVariantAttributeSet'
            )
        );
    }
    
    /**
     * Decorates the getCMSFields method and adds variant specific GUI elements.
     *
     * @param FieldSet &$fields The FieldSet from the decorated object.
     * 
     * @return void
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 24.05.2012
     */
    public function updateCMSFields(FieldSet &$fields) {
        if ($this->owner->ID > 0) {
            $fields->removeByName('SilvercartAttributedVariantAttributeSets');
            $fields->removeByName('SilvercartProductVariantAttributeSets');
            $fields->findOrMakeTab('Root.variantTabRoot', _t('SilvercartProductVariantProduct.TAB_VARIANT_LABEL'));
            $fields->addFieldToTab(
                    'Root.variantTabRoot',
                    new SilvercartVariantTableListField(
                        $this->owner,
                        'SilvercartVariantAttributeSets',
                        'SilvercartVariantAttributeSet'
                    )
            );
        }
    }

    /**
     * Returns whether this product has variants or not
     * 
     * @return boolean
     * 
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 30.11.2012
     */
    public function hasProductVariants() {
        if (is_null($this->hasVariants)) {
            $this->hasVariants = false;
            $variantSets       = $this->owner->SilvercartAttributedVariantAttributeSets();
            if (!is_null($variantSets) &&
                $variantSets->Count() > 0) {
                $this->hasVariants = true;
            }
        }
        return $this->hasVariants;
    }
    
    /**
     * Adds a product to the SilvercartShoppingCart and attaches the given
     * attributeSets to the position.
     *
     * @param int   $cartID              ID of the users shopping cart
     * @param int   $quantity            Amount of products to be added
     * @param array $attributes          The attributes that shall be attached to the created position
     * @param array $userInputAttributes Optional: the user generated attributes that shall be attached to the created position
     *
     * @return SilvercartShoppingCartPosition
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 30.11.2012
     */
    public function SVTLFaddToCartWithAttributes($cartID, $quantity = 1, $attributes, $userInputAttributes =  false) {
        // --------------------------------------------------------------------
        // Get user generated attributes
        // --------------------------------------------------------------------
        if ($userInputAttributes === false) {
            $userInputAttributes = array();

            foreach ($this->owner->SilvercartAttributedVariantAttributeSets() as $attributedAttributeSet) {
                if ($attributedAttributeSet->SilvercartProductVariantAttributeSet()->type == 'userInput') {

                    $dataSourceClassName = $attributedAttributeSet->SilvercartProductVariantAttributeSet()->dataSourceClass;

                    if (class_exists($dataSourceClassName)) {
                        $dataSourceClass = call_user_func(array($dataSourceClassName, 'getOrCreateFromRequest'));

                        if ($dataSourceClass) {
                            $existingGeneratedAttribute = DataObject::get_one(
                                'SilvercartProductVariantUserGeneratedAttribute',
                                sprintf(
                                    "inputObjectClassName = '%s' AND inputObjectID = '%s'",
                                    $dataSourceClass->class,
                                    $dataSourceClass->ID
                                )
                            );

                            if ($existingGeneratedAttribute) {
                                $generatedAttribute = $existingGeneratedAttribute;
                            } else {
                                $generatedAttribute     = new SilvercartProductVariantUserGeneratedAttribute();
                                $generatedAttribute->setField('inputObjectClassName',   $dataSourceClass->class);
                                $generatedAttribute->setField('inputObjectID',          $dataSourceClass->ID);
                                $generatedAttribute->write();
                            }
                            $userInputAttributes[] = $generatedAttribute;
                        }
                    }
                }
            }
        }
        
        // --------------------------------------------------------------------
        // If no standard or user generated attributes were given we use the
        // standard addToCart mechanism.
        // --------------------------------------------------------------------
        if (empty($userInputAttributes) &&
            (!is_array($attributes) ||
              count($attributes) == 0)
           ) {
            return $this->owner->addToCart($cartID, $quantity);
        }
        
        // --------------------------------------------------------------------
        // Generate filter for standard attributes
        // --------------------------------------------------------------------
        $attributeFilter        = '';
        $shoppingCartPosition   = false;
        $shoppingCartPositionID = false;
        $matchingAttributes     = 0;
        
        if ($quantity == 0 || $cartID == 0) {
            return false;
        }

        foreach ($attributes as $attributeSetID => $attribute) {
            if (empty($attributeFilter)) {
                $attributeFilter .= ' AND (';
            } else {
                $attributeFilter .= ' OR ';
            }
            $attributeFilter .= sprintf(
                "(CP_A.`SilvercartProductVariantAttributeSet` = %d AND CP_A.`SilvercartProductVariantAttributeID` = %d AND CP_A.`SilvercartShoppingCartPositionID` = CP.ID)",
                $attributeSetID,
                $attribute
            );
        }
        
        if (!empty($attributeFilter)) {
            $attributeFilter .= ')';
        }
        
        // --------------------------------------------------------------------
        // Generate filter for user generated attributes
        // --------------------------------------------------------------------
        $userAttributeFilter = '';
        
        foreach ($userInputAttributes as $userInputAttribute) {
            if (!empty($userAttributeFilter)) {
                $userAttributeFilter .= ' OR ';
            }
            
            $userAttributeFilter .= sprintf(
                "(UIA.inputObjectClassName = '%s' AND UIA.inputObjectID = %d)",
                $userInputAttribute->inputObjectClassName,
                $userInputAttribute->inputObjectID
            );
        }
        
        if (!empty($userAttributeFilter)) {
            $userAttributeFilter = ' AND ('.$userAttributeFilter.')';
        }
        
        // --------------------------------------------------------------------
        // Check if there's a position matching all given attributes
        // --------------------------------------------------------------------
        $records = DB::query(
            sprintf(
                "
                SELECT
                    CP.ID, COUNT(DISTINCT CP_A.SilvercartProductVariantAttributeSet) + COUNT(DISTINCT UIA.ID) AS MATCHING_ATTRIBUTES
                FROM
                    SilvercartShoppingCartPosition CP
                LEFT JOIN
                    SilvercartShoppingCartPosition_ProductVariantAttributes CP_A
                ON
                    CP_A.SilvercartShoppingCartPositionID = CP.ID
                LEFT JOIN
                    SilvercartShoppingCartPosition_ProductVariantUGAttributes CP_UIA
                ON
                    CP_UIA.SilvercartShoppingCartPositionID = CP.ID
                LEFT JOIN
                    SilvercartProductVariantUserGeneratedAttribute UIA
                ON
                    UIA.ID = CP_UIA.SilvercartProductVariantUserGeneratedAttributeID
                WHERE
                    CP.`SilvercartProductID` = %d AND
                    CP.`SilvercartShoppingCartID` = %d
                    %s
                    %s
                GROUP BY
                    CP.ID
                ",
                $this->owner->ID,
                $cartID,
                $attributeFilter,
                $userAttributeFilter
            )
        );
        
        if ($records) {
            $positionAttributeMatches = array();
            
            foreach ($records as $record) {
                if (!array_key_exists($record['ID'], $positionAttributeMatches)) {
                    $positionAttributeMatches[$record['ID']] = $record['MATCHING_ATTRIBUTES'];
                }
            }
            
            foreach ($positionAttributeMatches as $positionID => $matchingAttributes) {
                if ($matchingAttributes == (count($attributes) + count($userInputAttributes))) {
                    $shoppingCartPositionID = $positionID;
                    break;
                }
            }
        }
        
        if ($shoppingCartPositionID !== false) {
            $shoppingCartPosition = DataObject::get_by_id('SilvercartShoppingCartPosition', $shoppingCartPositionID);
        }

        // --------------------------------------------------------------------
        // Create a new shopping cart position if none was found
        // --------------------------------------------------------------------
        if (!$shoppingCartPosition) {
            $shoppingCartPosition = new SilvercartShoppingCartPosition();
            
            $shoppingCartPosition->castedUpdate(
                array(
                    'SilvercartShoppingCartID' => $cartID,
                    'SilvercartProductID' => $this->owner->ID
                )
            );
            $shoppingCartPosition->write();
            $shoppingCartPosition = DataObject::get_by_id('SilvercartShoppingCartPosition', $shoppingCartPosition->ID);
            
            if ($shoppingCartPosition) {
                // ------------------------------------------------------------
                // Attach standard attributes
                // ------------------------------------------------------------
                foreach ($attributes as $attributeSetID => $attribute) {
                    $silvercartProductVariantAttribute = DataObject::get_by_id('SilvercartProductVariantAttribute', $attribute);

                    if ($silvercartProductVariantAttribute) {
                        DB::query(
                            sprintf(
                                "
                                INSERT INTO
                                    SilvercartShoppingCartPosition_ProductVariantAttributes (
                                        SilvercartShoppingCartPositionID,
                                        SilvercartProductVariantAttributeID,
                                        SilvercartProductVariantAttributeSet
                                    )
                                VALUES (
                                        %d,
                                        %d,
                                        %d
                                    )
                                ",
                                $shoppingCartPosition->ID,
                                $attribute,
                                $attributeSetID
                            )
                        );
                    }
                }
                
                // ------------------------------------------------------------
                // Attach user generated attributes
                // ------------------------------------------------------------
                foreach ($userInputAttributes as $attribute) {
                    DB::query(
                        sprintf(
                            "
                            INSERT INTO
                                SilvercartShoppingCartPosition_ProductVariantUGAttributes (
                                    SilvercartShoppingCartPositionID,
                                    SilvercartProductVariantUserGeneratedAttributeID
                                )
                            VALUES (
                                    %d,
                                    %d
                                )
                            ",
                            $shoppingCartPosition->ID,
                            $attribute->ID
                        )
                    );
                }
            }
        }
        
        // --------------------------------------------------------------------
        // Add quantity to the shopping cart position
        // --------------------------------------------------------------------
        if ($shoppingCartPosition->isQuantityIncrementableBy($quantity)) {
            $shoppingCartPosition->Quantity += $quantity;
        } else {
            if ($this->owner->StockQuantity > 0) {
                $shoppingCartPosition->Quantity += $this->owner->StockQuantity - $shoppingCartPosition->Quantity;
                $shoppingCartPosition->write(); //we have to write because we need the ID
                SilvercartShoppingCartPositionNotice::setNotice($shoppingCartPosition->ID, "remaining");  
            } else {
                return false;
            }
        }
        $shoppingCartPosition->write();
        
        return $shoppingCartPosition;
    }
    
    /**
     * Delete all attributed variant sets and their attributes.
     *
     * @return void
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 02.01.2012
     */
    public function updateOnBeforeDelete() {
        foreach ($this->owner->SilvercartAttributedVariantAttributeSets() as $attributedAttributeSet) {
            $attributedAttributeSet->delete();
        }
    }
}

/**
 * Extends the SilvercartProduct_CollectionController by variant specific
 * attributes and methods.
 *
 * @package SilvercartProductVariants
 * @subpackage Products
 * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
 * @since 11.11.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @copyright 2011 pixeltricks GmbH
 */
class SilvercartProductVariantProduct_RecordController extends DataObjectDecorator {

    /**
     * Adds the abillity to execute additional actions to the model admin's
     * action handling.
     *
     * @param SS_HTTPRequest $request The request object
     *
     * @return mixed
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 15.03.2012
     */
    public function handleAction(SS_HTTPRequest $request) {
        $allRequestParams = $request->allParams();

        if (array_key_exists('ID', $allRequestParams)) {
            $action = str_replace('action_', '', $allRequestParams['ID']);
        }
        if (array_key_exists('OtherID', $allRequestParams)) {
            $otherId = $allRequestParams['OtherID'];
        }

        switch ($action) {
            case 'doSVTLFAddUnAssignedAttributeSet':
                return $this->doSVTLFAddUnAssignedAttributeSet($request, $otherId);
                break;
            case 'doSVTLFRemoveAssignedAttributeSet':
                return $this->doSVTLFRemoveAssignedAttributeSet($request, $otherId);
                break;
            case 'doSVTLFDeactivateAttributeInSet':
                return $this->doSVTLFDeactivateAttributeInSet($request, $otherId);
                break;
            case 'doSVTLFActivateAttributeInSet':
                return $this->doSVTLFActivateAttributeInSet($request, $otherId);
                break;
            case 'doSVTLFSetDefault':
                return $this->doSVTLFSetDefault($request, $otherId);
                break;
            case 'doSVTLFRemoveDefault':
                return $this->doSVTLFRemoveDefault($request, $otherId);
                break;
            case 'SVTLFsaveFieldModifier':
                return $this->SVTLFsaveFieldModifier($request, $otherId);
                break;
            case 'SVTLFaddFieldModifier':
                return $this->SVTLFaddFieldModifier($request, $otherId);
                break;
            case 'SVTLFremoveFieldModifier':
                return $this->SVTLFremoveFieldModifier($request, $otherId);
                break;
            case 'doSVTLFActivateAllAttributes':
                return $this->doSVTLFActivateAllAttributes($request, $otherId);
                break;
            case 'doSVTLFDeactivateAllAttributes':
                return $this->doSVTLFDeactivateAllAttributes($request, $otherId);
                break;
            case 'SVTLFsaveSettings':
                return $this->doSVTLFSaveSettings($request, $otherId);
                break;
            default:
                return false;
        }
    }

    /**
     * Activates a variant.
     *
     * @param SS_HTTPRequest $request   The request object
     * @param int            $variantId The ID of the variant
     *
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 15.11.2011
     */
    public function doSVTLFActivateAttributeInSet(SS_HTTPRequest $request, $variantId) {
        $silvercartProduct = $this->owner->currentRecord;
        DB::query(
            sprintf(
                "
                UPDATE
                    SilvercartAttributedVariantAttributeSet_Attributes
                SET
                    isActive = 1
                WHERE
                    ID = %d
                ",
                $variantId
            )
        );
        $silvercartProduct->write();

        return $this->returnTableFieldForAjaxRequests(
            $this->getAttributeSetIdByAttributedSetId($variantId)
        );
    }

    /**
     * Activates all variants in the given set.
     *
     * @param SS_HTTPRequest $request   The request object
     * @param int            $variantId The ID of the variant
     *
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 16.02.2012
     */
    public function doSVTLFActivateAllAttributes(SS_HTTPRequest $request, $variantId) {
        $attributedSet = DataObject::get_by_id('SilvercartAttributedVariantAttributeSet', $variantId);

        if ($attributedSet) {
            $silvercartProduct = $this->owner->currentRecord;
            DB::query(
                sprintf(
                    "
                    UPDATE
                        SilvercartAttributedVariantAttributeSet_Attributes
                    SET
                        isActive = 1
                    WHERE
                        SilvercartAttributedVariantAttributeSetID = %d
                    ",
                    $variantId
                )
            );
            $silvercartProduct->write();
        }

        return $this->returnTableFieldForAjaxRequests($variantId);
    }

    /**
     * Deactivates all variants in the given set.
     *
     * @param SS_HTTPRequest $request   The request object
     * @param int            $variantId The ID of the variant
     *
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 16.02.2012
     */
    public function doSVTLFDeactivateAllAttributes(SS_HTTPRequest $request, $variantId) {
        $attributedSet = DataObject::get_by_id('SilvercartAttributedVariantAttributeSet', $variantId);

        if ($attributedSet) {
            $silvercartProduct = $this->owner->currentRecord;
            DB::query(
                sprintf(
                    "
                    UPDATE
                        SilvercartAttributedVariantAttributeSet_Attributes
                    SET
                        isActive = 0
                    WHERE
                        SilvercartAttributedVariantAttributeSetID = %d
                    ",
                    $variantId
                )
            );
            $silvercartProduct->write();
        }

        return $this->returnTableFieldForAjaxRequests($variantId);
    }

    /**
     * Deactivates a variant.
     *
     * @param SS_HTTPRequest $request   The request object
     * @param int            $variantId The ID of the variant
     *
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 15.11.2011
     */
    public function doSVTLFDeactivateAttributeInSet(SS_HTTPRequest $request, $variantId) {
        $silvercartProduct = $this->owner->currentRecord;
        DB::query(
            sprintf(
                "
                UPDATE
                    SilvercartAttributedVariantAttributeSet_Attributes
                SET
                    isActive = 0
                WHERE
                    ID = %d
                ",
                $variantId
            )
        );
        $silvercartProduct->write();

        return $this->returnTableFieldForAjaxRequests(
            $this->getAttributeSetIdByAttributedSetId($variantId)
        );
    }

    /**
     * Adds an unassigned attribute set to a product.
     *
     * @param SS_HTTPRequest $request        The request object
     * @param int            $attributeSetId The ID of the attributeset
     *
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 11.11.2011
     */
    public function doSVTLFAddUnAssignedAttributeSet(SS_HTTPRequest $request, $attributeSetId) {
        $attributeSet = DataObject::get_by_id('SilvercartProductVariantAttributeSet', $attributeSetId);

        if ($attributeSet) {
            $silvercartProduct = $this->owner->currentRecord;

            if (!$silvercartProduct->SilvercartAttributedVariantAttributeSets()->find('SilvercartProductVariantAttributeSetID', $attributeSet->ID)) {
                $silvercartAttributedVariantAttributeSet = new SilvercartAttributedVariantAttributeSet();
                $silvercartAttributedVariantAttributeSet->setField('SilvercartProductID',                       $silvercartProduct->ID);
                $silvercartAttributedVariantAttributeSet->setField('SilvercartProductVariantAttributeSetID',    $attributeSetId);
                $silvercartAttributedVariantAttributeSet->write();

                $silvercartProduct->SilvercartAttributedVariantAttributeSets()->add($silvercartAttributedVariantAttributeSet);

                // Create attributed attribute relation
                foreach ($attributeSet->Attributes() as $attribute) {
                    DB::query(
                        sprintf(
                            "
                            INSERT INTO
                                SilvercartAttributedVariantAttributeSet_Attributes (
                                    SilvercartAttributedVariantAttributeSetID,
                                    SilvercartProductVariantAttributeID
                                )
                            VALUES (
                                    %d,
                                    %d
                                )
                            ",
                            $silvercartAttributedVariantAttributeSet->ID,
                            $attribute->ID
                        )
                    );
                }
            }
        }
        $silvercartProduct->write();

        return $this->returnTableFieldForAjaxRequests($silvercartAttributedVariantAttributeSet->ID);
    }

    /**
     * Removes an assigned attribute set to a product.
     *
     * @param SS_HTTPRequest $request                  The request object
     * @param int            $attributedAttributeSetId The ID of the attributedAttributeSet
     *
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 11.11.2011
     */
    public function doSVTLFRemoveAssignedAttributeSet(SS_HTTPRequest $request, $attributedAttributeSetId) {
        $silvercartProduct      = $this->owner->currentRecord;
        $attributedAttributeSet = DataObject::get_by_id('SilvercartAttributedVariantAttributeSet', $attributedAttributeSetId);

        if ($attributedAttributeSet) {
            $attributeSet = DataObject::get_by_id('SilvercartProductVariantAttributeSet', $attributedAttributeSet->SilvercartProductVariantAttributeSet()->ID);

            if ($attributeSet) {
                $silvercartProduct->SilvercartProductVariantAttributeSets()->remove($attributeSet);

                DB::query(
                    sprintf(
                        "
                        DELETE FROM
                            SilvercartAttributedVariantAttributeSet
                        WHERE
                            SilvercartProductID = %d AND
                            ID = %d
                        ",
                        $silvercartProduct->ID,
                        $attributedAttributeSetId
                    )
                );

                DB::query(
                    sprintf(
                        "
                        DELETE FROM
                            SilvercartAttributedVariantAttributeSet_Attributes
                        WHERE
                            SilvercartAttributedVariantAttributeSetID = %d
                        ",
                        $attributedAttributeSetId
                    )
                );
            }
            $silvercartProduct->write();
        }

        return $this->returnTableFieldForAjaxRequests();
    }

    /**
     * Saves the settings
     *
     * @param SS_HTTPRequest $request   The request object
     * @param int            $variantId The ID of the variant
     *
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 08.03.2012
     */
    public function doSVTLFSaveSettings(SS_HTTPRequest $request, $variantId) {
        $variantConfig = DataObject::get_one('SilvercartProductVariantConfiguration');

        if ($variantConfig) {
            if (array_key_exists('hideDeactivatedAttributes', $request->postVars())) {
                $variantConfig->hideDeactivatedAttributes = true;
            } else {
                $variantConfig->hideDeactivatedAttributes = false;
            }
            $variantConfig->write();
        }

        return $this->returnTableFieldForAjaxRequests($variantId);
    }

    /**
     * Makes a variant the default.
     *
     * @param SS_HTTPRequest $request   The request object
     * @param int            $variantId The ID of the variant
     *
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 08.12.2011
     */
    public function doSVTLFSetDefault(SS_HTTPRequest $request, $variantId) {
        $silvercartProduct = $this->owner->currentRecord;
        $attributeSetID = false;
        $records        = DB::query(
            sprintf(
                "
                SELECT
                    SilvercartAttributedVariantAttributeSetID
                FROM
                    SilvercartAttributedVariantAttributeSet_Attributes
                WHERE
                    ID = %d
                ",
                $variantId
            )
        );

        if ($records) {
            foreach ($records as $record) {
                $attributeSetID = $record['SilvercartAttributedVariantAttributeSetID'];
            }
        }

        if ($attributeSetID !== false) {
            DB::query(
                sprintf(
                    "
                    UPDATE
                        SilvercartAttributedVariantAttributeSet_Attributes
                    SET
                        isDefault = 0
                    WHERE
                        SilvercartAttributedVariantAttributeSetID = %d
                    ",
                    $attributeSetID
                )
            );
            DB::query(
                sprintf(
                    "
                    UPDATE
                        SilvercartAttributedVariantAttributeSet_Attributes
                    SET
                        isDefault = 1
                    WHERE
                        ID = %d
                    ",
                    $variantId
                )
            );
        }
        $silvercartProduct->write();

        return $this->returnTableFieldForAjaxRequests($attributeSetID);
    }

    /**
     * Resets the default attribute.
     *
     * @param SS_HTTPRequest $request   The request object
     * @param int            $variantId The ID of the variant
     *
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 07.03.2012
     */
    public function doSVTLFRemoveDefault(SS_HTTPRequest $request, $variantId) {
        $attributedSet = DataObject::get_by_id('SilvercartAttributedVariantAttributeSet', $variantId);

        if ($attributedSet) {
            $silvercartProduct = $this->owner->currentRecord;
            DB::query(
                sprintf(
                    "
                    UPDATE
                        SilvercartAttributedVariantAttributeSet_Attributes
                    SET
                        isDefault = 0
                    WHERE
                        SilvercartAttributedVariantAttributeSetID = %d
                    ",
                    $variantId
                )
            );
            $silvercartProduct->write();
        }

        return $this->returnTableFieldForAjaxRequests($variantId);
    }

    /**
     * Saves field modifier values.
     *
     * @param SS_HTTPRequest $request         The request object
     * @param int            $fieldModifierId The ID of the field modifier
     *
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 18.01.2012
     */
    public function SVTLFsaveFieldModifier(SS_HTTPRequest $request, $fieldModifierId) {
        $attributedSetId = null;

        if (array_key_exists('asid', $_REQUEST)) {
            $attributedSetId = Convert::raw2sql($_REQUEST['asid']);
        }

        $fieldModifier = DataObject::get_by_id('SilvercartProductVariantFieldModifier', Convert::raw2sql($fieldModifierId));

        if ($fieldModifier) {
            $fieldModifier->setField('silvercartProductFieldName', $_REQUEST['fieldModifier_fieldName']);
            $fieldModifier->setField('modifierAction',             $_REQUEST['fieldModifier_fieldAction']);
            $fieldModifier->setField('modifierValue',              $_REQUEST['fieldModifier_fieldValue']);
            $fieldModifier->write();

            $silvercartProduct = $this->owner->currentRecord;
            $silvercartProduct->write();
        }

        return $this->returnTableFieldForAjaxRequests($attributedSetId);
    }

    /**
     * Add a field modifier.
     *
     * @param SS_HTTPRequest $request    The request object
     * @param int            $relationId The ID of the relation table
     *
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 18.01.2012
     */
    public function SVTLFaddFieldModifier(SS_HTTPRequest $request, $relationId) {
        $silvercartProduct = $this->owner->currentRecord;
        $attributeId    = 0;
        $attributeSetId = null;
        $relationId     = (int) $relationId;

        $records = DB::query(
            sprintf(
                "
                SELECT
                    SilvercartProductVariantAttributeID,
                    SilvercartAttributedVariantAttributeSetID
                FROM
                    SilvercartAttributedVariantAttributeSet_Attributes
                WHERE
                    ID = %d
                LIMIT 1
                ",
                $relationId
            )
        );

        if ($records) {
            foreach ($records as $record) {
                $attributeId                     = $record['SilvercartProductVariantAttributeID'];
                $attributedVariantAttributeSetID = $record['SilvercartAttributedVariantAttributeSetID'];
                break;
            }
        }

        if ($attributeId > 0) {
            $fieldModifier = new SilvercartProductVariantFieldModifier();
            $fieldModifier->setField('SilvercartProductVariantAttributeID',       $attributeId);
            $fieldModifier->setField('SilvercartAttributedVariantAttributeSetID', $attributedVariantAttributeSetID);
            $fieldModifier->setField('silvercartProductFieldName', SilvercartProductVariantFieldModifier::getDefaultFieldName());
            $fieldModifier->setField('modifierAction',             SilvercartProductVariantFieldModifier::getDefaultFieldModifierAction());
            $fieldModifier->setField('modifierValue',              SilvercartProductVariantFieldModifier::getDefaultFieldModifierValue());
            $fieldModifier->write();
        }
        $silvercartProduct->write();

        return $this->returnTableFieldForAjaxRequests(
            $this->getAttributeSetIdByAttributedSetId($relationId)
        );
    }

    /**
     * Remove a field modifier.
     *
     * @param SS_HTTPRequest $request         The request object
     * @param int            $fieldModifierId The ID of the field modifier
     *
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 18.01.2012
     */
    public function SVTLFremoveFieldModifier(SS_HTTPRequest $request, $fieldModifierId) {
        $silvercartProduct = $this->owner->currentRecord;
        $attributedSetId = null;

        if (array_key_exists('asid', $_REQUEST)) {
            $attributedSetId = Convert::raw2sql($_REQUEST['asid']);
        }

        $fieldModifier = DataObject::get_by_id('SilvercartProductVariantFieldModifier', $fieldModifierId);

        if ($fieldModifier) {
            $fieldModifier->SilvercartProductVariantAttribute()->SilvercartProductVariantFieldModifiers()->remove($fieldModifier);
            $fieldModifier->delete();
        }
        $silvercartProduct->write();

        return $this->returnTableFieldForAjaxRequests($attributedSetId);
    }

    /**
     * Returns the attributeSetId for the given attributedAttributeSetId.
     *
     * @param int $variantId The ID of the attributed attribute set
     *
     * @return int
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 15.02.2012
     */
    public function getAttributeSetIdByAttributedSetId($variantId) {
        $attributeSetID = null;
        $records        = DB::query(
            sprintf(
                "
                SELECT
                    SilvercartAttributedVariantAttributeSetID
                FROM
                    SilvercartAttributedVariantAttributeSet_Attributes
                WHERE
                    ID = %d
                ",
                $variantId
            )
        );

        if ($records) {
            foreach ($records as $record) {
                $attributeSetID = $record['SilvercartAttributedVariantAttributeSetID'];
                break;
            }
        }

        return $attributeSetID;
    }

    /**
     * Returns the SilvercartVariantTableListField for ajax requests as HTML
     * code.
     *
     * @param int $activeAttributeSetId The ID of the active attribute set
     *
     * @return string
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 24.05.2012
     */
    protected function returnTableFieldForAjaxRequests($activeAttributeSetId = null) {
        $data = SilvercartVariantTableListField::getTableFieldData($this->owner->currentRecord, $activeAttributeSetId);
        return $this->owner->customise($data)->renderWith('SilvercartVariantTableListField');
    }
    
}
